﻿'***************************************************************************************************
'   未更新終了チェック関連クラス                          clsCheckNotUpdate3(Class)
'
'   ※登録ボタンをClickせずに終了しようとする時の警告表示対応の変数、共通プロシージャ
'     ⇒本クラスはフォームのNewイベントでInitializeComponentの直後に実装すること
'     ⇒ダブルクリック抑制クラスは｢clsOmitDoubleClick2(新)｣を使用すること
'
'   ※未更新終了チェックから除外したいコントロールがある場合の処置
'   　[例]
'   　和暦表示に切り替えられるチェックボックスを配置したフォームだと
'   　そのチェックボックス自体と日付表示のテキストボックスでは西暦／和暦の変更だけでは
'   　日付自体を変更したわけではないので未更新終了チェックの対象にはしたくない
'   　このような場合は以下のように処置する
'   　①本処理の初期化時点で第3引数にチェック除外対象のコントロール名を配列にしてセットする
'   　　⇒この例だとチェックボックスとテキストボックスの両方
'   　②マスク状態変更の処理中を判定するスイッチをモジュールレベルに新設する
'   　③チェックボックスの変更イベントでテキストボックスを操作する前に上記スイッチにTrueをセット
'   　④　　〃　　　　　　　　　　　　　テキストボックスを操作した後に上記スイッチにFalseをセット
'   　⑤テキストボックス変更イベントで上記スイッチがFalseなら本クラスのNotUpdateプロパティにTrueをセット
'   　⇒パスワードのマスク解除チェックボックスの場合ではTextChangedイベントは発生しないので
'   　　チェックボックスのみを①の第3引数にセットするだけで良い
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev     変更履歴内容-------------------------------------------------------------------->
' 12/09/19(1.0.0.0)新規作成
' 13/01/04(1.1.0.0)独自メッセージボックス(clsMessageBox2)への変更対応
' 13/01/17(1.1.1.0)未更新判定でキャンセルされた時元フォームの戻り値をクリアする機能を追加
' 13/07/06(1.1.2.0)NumericUpDownコントロールの対応を追加
' 14/07/07(1.1.2.1)GP_ControlsAddHandlerでコンボのSelectedIndexChangedの対応を追加
' 14/09/20(1.1.2.2)GP_ControlsAddHandlerでUserControl(コンテナ扱い)等の対応を追加
' 15/01/29(1.1.3.0)GP_ControlsAddHandlerにRichTextBoxの処置を追加
' 15/02/23(1.1.3.1)ComboBoxの変異イベントを修正(SelectedIndexChanged､TextChanged併存化)
' 16/07/12(1.2.0.0)Disposeを実装する対応(イベントハンドラ開放含む)
' 16/10/01(1.2.1.0)DateTimePickerのイベントを追加する対応
' 17/02/04(1.3.0.0)clsCheckNotUpdate3として新規作成､g_strFatalErrMSG⇒swFatalErrorの変更
' 17/02/04(1.3.0.0)Disposeの撤去
' 17/11/11(1.3.1.0)ダブルクリック抑制クラス(clsOmitDoubleClick2)のClosingスイッチ解除対応を追加
' 18/04/07(1.3.2.0)当クラスで変異イベントを検知しないコントロールの指定機能を追加する対応(Option)
'***************************************************************************************************
Friend Class clsCheckNotUpdate3
    '-----------------------------------------------------------------------------------------------
    Private g_objOwnerForm As Object = Nothing                  ' 現在フォーム
    Private g_objOmitDoubleClick As clsOmitDoubleClick2         ' ダブルクリック抑制クラス
    Private g_cnsProgTitle As String = "未更新終了チェック"     ' フォームタイトル
    Private g_swNotUpdate As Boolean = False                    ' 未登録終了警告スイッチ
    Private g_swFatalError As Boolean = False                   ' 致命例外判定スイッチ
    Private g_swLockItem As Boolean = False                     ' 項目ロックスイッチ
    Private g_strNotUpdateMSG As String = "「キャンセル」が選択されました。" & ControlChars.CrLf & _
        "入力途中の内容は更新されませんが終了しますか？"        ' 未更新終了警告メッセージ
    Private g_strNotUpdateMSG_Tmp As String = ""                ' 〃(当回のみ)
    ' 当クラスで変異イベントを検知しないコントロール名テーブル
    Private g_tblOmitControlNm() As String = Nothing            ' 除外コントロール名テーブル

    '***********************************************************************************************
    ' ■■■ クラス初期化メソッド ■■■
    '***********************************************************************************************
    '* 処理名　：New
    '* 機能　　：初期化
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = Ownerフォーム(Object)
    '* 　　　　　Arg2 = ダブルクリック抑制クラス(Object)
    '* 　　　　　Arg3 = 除外コントロール名テーブル(Array:String)      ※Option
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2012年09月19日
    '* 作成者　：井上　治
    '* 更新日　：2018年04月07日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub New(ByRef objOwner As Object, _
                   ByRef objOmitDoubleClick As clsOmitDoubleClick2, _
                   Optional ByVal tblOmitControlNm() As String = Nothing)
        '-------------------------------------------------------------------------------------------
        g_objOwnerForm = objOwner                       ' Ownerフォーム
        g_cnsProgTitle = objOwner.Text                  ' プログラムタイトル
        g_objOmitDoubleClick = objOmitDoubleClick       ' ダブルクリック抑制クラス
        ' 除外コントロール名テーブル(Option)の受け取り
        If tblOmitControlNm IsNot Nothing Then
            g_tblOmitControlNm = tblOmitControlNm.Clone
        Else
            ReDim g_tblOmitControlNm(-1)
        End If
        '-------------------------------------------------------------------------------------------
        ' 未登録終了警告スイッチ対応イベントハンドラ追加
        Call GP_ControlsAddHandler(objOwner)
        ' FormClosingイベントハンドラ追加
        AddHandler CType(objOwner, Form).FormClosing, AddressOf Form_FormClosing
        '-------------------------------------------------------------------------------------------
        g_swFatalError = False
        g_swNotUpdate = False
    End Sub

    '***********************************************************************************************
    ' ■■■ サブ処理 ■■■
    '***********************************************************************************************
    '* 処理名　：GP_ControlsAddHandler
    '* 機能　　：入力コントロールへのイベントハンドラ追加処理
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 所属するコンテナ(Object)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2012年09月19日
    '* 作成者　：井上　治
    '* 更新日　：2018年04月07日
    '* 更新者　：井上　治
    '* 機能説明：コンテナ上の各入力コントロールにイベントハンドラを追加(変異検出用)
    '* 注意事項：本処理は再帰動作
    '***********************************************************************************************
    Private Sub GP_ControlsAddHandler(ByVal objContainer As Object)
        '-------------------------------------------------------------------------------------------
        ' コントロールの種別によって変異検出用のイベントハンドラを追加
        For Each objCont As Control In objContainer.Controls
            ' 除外コントロール名テーブルに無ければ変異イベントを設定
            If Array.IndexOf(g_tblOmitControlNm, objCont.Name) < 0 Then
                ' コントロールタイプの分岐
                If TypeOf objCont Is TextBox Then
                    ' テキストボックス
                    AddHandler CType(objCont, TextBox).TextChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is ComboBox Then
                    ' コンボボックス
                    With CType(objCont, ComboBox)
                        AddHandler .SelectedIndexChanged, AddressOf Controls_ValueChanged
                        AddHandler .TextChanged, AddressOf Controls_ValueChanged
                    End With
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is CheckBox Then
                    ' チェックボックス
                    AddHandler CType(objCont, CheckBox).CheckedChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is RadioButton Then
                    ' ラジオボタン
                    AddHandler CType(objCont, RadioButton).CheckedChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is DataGridView Then
                    ' DataGridView
                    AddHandler CType(objCont, DataGridView).CellValueChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is MaskedTextBox Then
                    ' マスクドテキストボックス(DatePickerJP等)
                    AddHandler CType(objCont, MaskedTextBox).TextChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is RichTextBox Then
                    ' リッチテキストボックス
                    AddHandler CType(objCont, RichTextBox).TextChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is ListBox Then
                    ' リストボックス
                    AddHandler CType(objCont, ListBox).SelectedIndexChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is NumericUpDown Then
                    ' ニューメリックアップダウン
                    AddHandler CType(objCont, NumericUpDown).ValueChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf TypeOf objCont Is DateTimePicker Then
                    ' DateTimePicker
                    AddHandler CType(objCont, DateTimePicker).ValueChanged, AddressOf Controls_ValueChanged
                    '-------------------------------------------------------------------------------
                ElseIf ((TypeOf objCont Is TabControl) OrElse _
                        (TypeOf objCont Is TabPage) OrElse _
                        (TypeOf objCont Is GroupBox) OrElse _
                        (TypeOf objCont Is Panel) OrElse _
                        (TypeOf objCont Is TableLayoutPanel) OrElse _
                        (TypeOf objCont Is FlowLayoutPanel) OrElse _
                        (TypeOf objCont Is SplitContainer) OrElse _
                        (TypeOf objCont Is UserControl)) Then
                    ' コンテナ時は再帰呼び出し
                    Call GP_ControlsAddHandler(objCont)
                    '-------------------------------------------------------------------------------
                End If
            End If
        Next objCont
    End Sub

    '***********************************************************************************************
    '   ■■■ イベント
    '***********************************************************************************************
    '* 処理名　：Controls_ValueChanged
    '* 機能　　：各入力コントロールの値変更イベント処理
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2012年09月19日
    '* 作成者　：井上　治
    '* 更新日　：2014年07月07日
    '* 更新者　：井上　治
    '* 機能説明：どれかの入力コントロールで更新入力があったかを判断するイベント
    '* 注意事項：Form_Loadイベントで全入力コントロールのイベントハンドラ追加を行なう前提
    '***********************************************************************************************
    Friend Sub Controls_ValueChanged(ByVal sender As Object, ByVal e As System.EventArgs)
        '-------------------------------------------------------------------------------------------
        ' 未登録終了警告スイッチをセット
        If Not g_swNotUpdate Then
            g_swNotUpdate = True
        End If
    End Sub

    '***********************************************************************************************
    '* 処理名　：Form_FormClosing
    '* 機能　　：フォーム閉じる動作(FormClosing)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2012年09月19日
    '* 作成者　：井上　治
    '* 更新日　：2017年11月11日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub Form_FormClosing(ByVal sender As Object, _
                                ByVal e As FormClosingEventArgs)
        '-------------------------------------------------------------------------------------------
        ' 未登録終了警告スイッチの判定
        e.Cancel = False
        If (g_swNotUpdate AndAlso Not g_swLockItem AndAlso Not g_swFatalError) Then
            Dim strMSG As String = g_strNotUpdateMSG
            If g_strNotUpdateMSG_Tmp.Length <> 0 Then
                strMSG = g_strNotUpdateMSG_Tmp
                g_strNotUpdateMSG_Tmp = String.Empty
            End If
            ' 終了しますかの確認メッセージ
            If MessageBox2.DialogShow( _
                g_objOwnerForm, _
                strMSG, _
                g_cnsProgTitle, _
                MessageBoxButtons.YesNo, _
                MessageBoxIcon.Information) <> Windows.Forms.DialogResult.Yes Then
                ' ダブルクリック抑制クラスのClosingスイッチ解除
                g_objOmitDoubleClick.Closing = False
                ' Closeをキャンセル
                e.Cancel = True
                Exit Sub
            End If
        End If
        g_strNotUpdateMSG_Tmp = String.Empty
    End Sub

    '***********************************************************************************************
    '   ■■■ 参照・設定プロパティ
    '***********************************************************************************************
    '   未登録終了警告スイッチ(Boolean)
    '-----------------------------------------------------------------------------------------------
    Friend Property NotUpdate() As Boolean
        Get
            Return g_swNotUpdate
        End Get
        Set(ByVal value As Boolean)
            g_swNotUpdate = value
        End Set
    End Property

    '===============================================================================================
    '   致命例外判定スイッチ(Boolean)
    '-----------------------------------------------------------------------------------------------
    Friend Property FatalError As Boolean
        Get
            Return g_swFatalError
        End Get
        Set(value As Boolean)
            g_swFatalError = value
        End Set
    End Property

    '===============================================================================================
    '   項目ロックスイッチ(Boolean)
    '-----------------------------------------------------------------------------------------------
    Friend WriteOnly Property LockItem() As Boolean
        Set(ByVal value As Boolean)
            g_swLockItem = value
        End Set
    End Property

    '===============================================================================================
    '   未登録終了警告メッセージ(String)
    '-----------------------------------------------------------------------------------------------
    Friend WriteOnly Property NotUpdateMSG() As String
        Set(ByVal value As String)
            g_strNotUpdateMSG = value
        End Set
    End Property

    '===============================================================================================
    '   未登録終了警告メッセージ：当回のみ(String)
    '-----------------------------------------------------------------------------------------------
    Friend WriteOnly Property NotUpdateMSG_Tmp() As String
        Set(ByVal value As String)
            g_strNotUpdateMSG_Tmp = value
        End Set
    End Property

    '---------------------------------------<< End of Source >>-------------------------------------
End Class

