﻿'***************************************************************************************************
'     フォーム位置・サイズの退避・復旧関数                modAdjustLocationSize(Module)
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev     変更履歴内容-------------------------------------------------------------------->
' 16/07/07(1.2.3.0)新規作成
' 16/07/10(1.2.3.1)15p程度のはみ出しを許容するように修正
' 18/12/16(1.2.4.0)GP_FormAdjustLocationSize2で親フォームの中央に位置させる修正
'***************************************************************************************************
Module modAdjustLocationSize
    '===============================================================================================
    Private Const g_cnsArrowPoint As Integer = 15                   ' はみ出し許容ポイント
    Private g_objDesigndSize As Size = New Size(800, 600)           ' デザイナ時点フォームサイズ

    '***********************************************************************************************
    '   ■■■ 公開プロシージャ ■■■
    '***********************************************************************************************
    '* 処理名　：GP_FormAdjustLocationSize1
    '* 機能　　：フォーム位置・サイズ制御
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 該当フォーム(Object)
    '* 　　　　　Arg2 = 設定退避位置(Point)
    '* 　　　　　Arg3 = 設定退避サイズ(Size)
    '* 　　　　　Arg4 = デザイン時サイズ(Size)      ※DPI更新後であること
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年07月07日
    '* 作成者　：井上　治
    '* 更新日　：2016年07月07日
    '* 更新者　：井上　治
    '* 機能説明：フォーム退避値がスクリーンから外れている時はメインスクリーン中央に戻す機能
    '* 注意事項：以下の条件の場合に使用できる
    '* 　　　　　・Windows上に直接出ることができるフォーム(MDI子フォーム等でない)
    '* 　　　　　・親ウィンドウを持たない(隠しベースフォームを除く)
    '* 　　　　　・Sizableフォーム
    '* 　　　　　・設定に位置・サイズを保持させており、この復旧のNewプロシージャの最後に行なう
    '***********************************************************************************************
    Friend Sub GP_FormAdjustLocationSize1(ByRef objForm As Form, _
                                          ByVal objSettingLocation As Point, _
                                          ByVal objSettingSize As Size, _
                                          ByVal objDesigndSize As Size)
        '-------------------------------------------------------------------------------------------
        g_objDesigndSize = objDesigndSize                           ' デザイナ時点フォームサイズ
        ' 退避位置未設定、フォーム起動位置手動以外は除外
        If Not FP_CheckEnableAdjust(objForm, objSettingLocation) Then Exit Sub
        '-------------------------------------------------------------------------------------------
        Dim objSize As Size = objSettingSize                        ' 設定サイズ
        ' 退避サイズ未設定の場合はデザインサイズを充当
        If ((objSize.Width = 0) AndAlso (objSize.Height = 0)) Then
            objSize = objDesigndSize
        End If
        Dim objLocation As Point = objSettingLocation               ' 設定位置
        Dim objMainScreen As Screen = Nothing                       ' メインスクリーン
        '-------------------------------------------------------------------------------------------
        ' 各スクリーンを巡回してどれかのスクリーン内に収まっているかを確認
        Dim blnOK As Boolean = FP_CheckFormInScreens(objLocation, objSize, objMainScreen) ' 正常判定
        ' どのスクリーンにも収まっていない場合はメインスクリーン内(中央)に移動
        If Not blnOK AndAlso objMainScreen IsNot Nothing Then
            With objMainScreen.WorkingArea
                objLocation = New Point(.Left + (.Width - objForm.Width) \ 2, _
                                        .Top + (.Height - objForm.Height) \ 2)
            End With
            ' サイズは対象フォームのサイズに戻す(変更しない)
            objSize = objForm.Size
        End If
        ' 位置・サイズを設定(サイズが後であること)
        objForm.Location = objLocation
        objForm.Size = objSize
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_FormAdjustLocationSize2
    '* 機能　　：フォーム位置・サイズ制御
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 該当フォーム(Object)
    '* 　　　　　Arg2 = 親フォーム(Object)
    '* 　　　　　Arg3 = 設定退避位置(Point)
    '* 　　　　　Arg4 = 設定退避サイズ(Size)
    '* 　　　　　Arg5 = デザイン時サイズ(Size)      ※DPI更新後であること
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年07月07日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：フォーム退避値がスクリーンから外れている時は親フォームの中央に戻す機能
    '* 注意事項：以下の条件の場合に使用できる
    '* 　　　　　・Windows上に直接出ることができるフォーム(MDI子フォーム等でない)
    '* 　　　　　・親ウィンドウを持つ(MDI子フォームの場合はMDI親フォームを指定すること)
    '* 　　　　　・Sizableフォーム
    '* 　　　　　・設定に位置・サイズを保持させており、この復旧のNewプロシージャの最後に行なう
    '***********************************************************************************************
    Friend Sub GP_FormAdjustLocationSize2(ByRef objForm As Form, _
                                          ByRef objOwnerForm As Form, _
                                          ByVal objSettingLocation As Point, _
                                          ByVal objSettingSize As Size, _
                                          ByVal objDesigndSize As Size)
        '-------------------------------------------------------------------------------------------
        ' 退避位置未設定、フォーム起動位置手動以外は除外
        If Not FP_CheckEnableAdjust(objForm, objSettingLocation) Then Exit Sub
        '-------------------------------------------------------------------------------------------
        Dim objSize As Size = objSettingSize                        ' 設定サイズ
        ' 退避サイズ未設定の場合はデザインサイズを充当
        If ((objSize.Width = 0) AndAlso (objSize.Height = 0)) Then
            objSize = objDesigndSize
        End If
        Dim objLocation As Point = objSettingLocation               ' 設定位置
        '-------------------------------------------------------------------------------------------
        ' 各スクリーンを巡回してどれかのスクリーン内に収まっているかを確認
        Dim blnOK As Boolean = FP_CheckFormInScreens(objLocation, objSize) ' 正常判定
        ' どのスクリーンにも収まっていない場合は親フォーム内(中央)に移動
        If (Not blnOK OrElse ((objLocation.X = 0) AndAlso (objLocation.Y = 0))) Then
            With objOwnerForm
                objLocation = New Point(.Left + (.Width - objForm.Width) \ 2, _
                                        .Top + (.Height - objForm.Height) \ 2)
            End With
            ' サイズは対象フォームのサイズに戻す(変更しない)
            objSize = objForm.Size
        End If
        ' 位置・サイズを設定(サイズが後であること)
        objForm.Location = objLocation
        objForm.Size = objSize
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_FormAdjustLocation1
    '* 機能　　：フォーム位置制御
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 該当フォーム(Object)
    '* 　　　　　Arg2 = 設定退避位置(Point)
    '* 　　　　　Arg3 = ログインフォーム利用(Boolean)
    '* 　　　　　Arg4 = 親フォームサイズ(Size)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年07月07日
    '* 作成者　：井上　治
    '* 更新日　：2016年07月07日
    '* 更新者　：井上　治
    '* 機能説明：フォーム退避値がスクリーンから外れている時はメインスクリーン中央に戻す機能
    '* 注意事項：以下の条件の場合に使用できる
    '* 　　　　　・Windows上に直接出ることができるフォーム(MDI子フォーム等でない)
    '* 　　　　　・親ウィンドウを持たない(隠しベースフォームを除く)
    '* 　　　　　・固定Sizeフォーム
    '* 　　　　　・ログインフォームで利用する場合はオプション項目を指定すること
    '* 　　　　　・設定に位置を保持させており、この復旧のNewプロシージャの最後に行なう
    '***********************************************************************************************
    Friend Sub GP_FormAdjustLocation1(ByRef objForm As Form, _
                                      ByVal objSettingLocation As Point, _
                                      ByVal blnLoginForm As Boolean, _
                                      ByVal objOwnerFormSize As Size)
        '-------------------------------------------------------------------------------------------
        ' 退避位置未設定、フォーム起動位置手動以外は除外
        If Not FP_CheckEnableAdjust(objForm, objSettingLocation) Then Exit Sub
        '-------------------------------------------------------------------------------------------
        Dim objSize As Size = objForm.Size                          ' 該当フォームサイズ
        Dim objLocation As Point = objSettingLocation               ' 設定位置
        Dim objBottomRight As Point = New Point(objLocation.X + objSize.Width, _
                                                objLocation.Y + objSize.Height) ' 右下位置
        Dim objMainScreen As Screen = Nothing                       ' メインスクリーン
        '-------------------------------------------------------------------------------------------
        ' 各スクリーンを巡回してどれかのスクリーン内に収まっているかを確認
        Dim blnOK As Boolean = FP_CheckFormInScreens(objLocation, objSize, objMainScreen) ' 正常判定
        ' どのスクリーンにも収まっていない場合はメインスクリーン内(中央)に移動
        If Not blnOK AndAlso objMainScreen IsNot Nothing Then
            With objMainScreen.WorkingArea
                objLocation = New Point(.Left + (.Width - objForm.Width) \ 2, _
                                        .Top + (.Height - objForm.Height) \ 2)
            End With
        ElseIf blnLoginForm Then
            ' ログインフォーム時は親フォームを配置したものとしてその中央へ移動
            objLocation.X += (objOwnerFormSize.Width - objForm.Width) \ 2
            objLocation.Y += (objOwnerFormSize.Height - objForm.Height) \ 2
        End If
        ' 位置を設定
        objForm.Location = objLocation
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_FormAdjustLocation2
    '* 機能　　：フォーム位置制御
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 該当フォーム(Object)
    '* 　　　　　Arg2 = 親フォーム(Object)
    '* 　　　　　Arg3 = 設定退避位置(Point)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年07月07日
    '* 作成者　：井上　治
    '* 更新日　：2016年07月07日
    '* 更新者　：井上　治
    '* 機能説明：フォーム退避値がスクリーンから外れている時は親フォームの中央に戻す機能
    '* 注意事項：以下の条件の場合に使用できる
    '* 　　　　　・Windows上に直接出ることができるフォーム(MDI子フォーム等でない)
    '* 　　　　　・親ウィンドウを持つ(隠しベースフォームを除く)
    '* 　　　　　・固定Sizeフォーム
    '* 　　　　　・設定に位置を保持させており、この復旧のNewプロシージャの最後に行なう
    '***********************************************************************************************
    Friend Sub GP_FormAdjustLocation2(ByRef objForm As Form, _
                                      ByRef objOwnerForm As Form, _
                                      ByVal objSettingLocation As Point)
        '-------------------------------------------------------------------------------------------
        ' 退避位置未設定、フォーム起動位置手動以外は除外
        If Not FP_CheckEnableAdjust(objForm, objSettingLocation) Then Exit Sub
        '-------------------------------------------------------------------------------------------
        Dim objSize As Size = objForm.Size                          ' 該当フォームサイズ
        Dim objLocation As Point = objSettingLocation               ' 設定位置
        Dim objBottomRight As Point = New Point(objLocation.X + objSize.Width, _
                                                objLocation.Y + objSize.Height) ' 右下位置
        '-------------------------------------------------------------------------------------------
        ' 各スクリーンを巡回してどれかのスクリーン内に収まっているかを確認
        Dim blnOK As Boolean = FP_CheckFormInScreens(objLocation, objSize) ' 正常判定
        ' どのスクリーンにも収まっていない場合は親フォーム内(中央)に移動
        If Not blnOK Then
            With objOwnerForm
                objLocation = New Point(.Left + (.Width - objForm.Width) \ 2, _
                                        .Top + (.Height - objForm.Height) \ 2)
            End With
        End If
        ' 位置を設定
        objForm.Location = objLocation
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_FormSaveSettings1
    '* 機能　　：フォーム位置・サイズ・ウィンドウ状態の退避
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 該当フォーム(Object)
    '* 　　　　　Arg2 = 設定退避位置(Point)
    '* 　　　　　Arg3 = 設定退避サイズ(Size)
    '* 　　　　　Arg4 = 設定退避ウィンドウ状態(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年07月07日
    '* 作成者　：井上　治
    '* 更新日　：2016年07月07日
    '* 更新者　：井上　治
    '* 機能説明：フォーム位置・サイズを設定に退避(最大化を伴う場合に使用)
    '* 注意事項：最大化時は原則として退避せずスクリーンが異なる時のみ代替値を退避させる
    '***********************************************************************************************
    Friend Sub GP_FormSaveSettings1(ByRef objForm As Form, _
                                    ByRef objSettingLocation As Point, _
                                    ByRef objSettingSize As Size, _
                                    ByRef intWindowState As Integer)
        '-------------------------------------------------------------------------------------------
        ' WindowStateによる判定
        Select Case objForm.WindowState
            Case FormWindowState.Minimized                          ' 最小化
                ' (何もしない)
            Case FormWindowState.Maximized                          ' 最大化
                ' サイズは退避しない(位置は前回退避値が同スクリーンでなければ代替値)
                objSettingLocation = FP_GetPositionDummy(objForm, _
                                                         objSettingLocation, _
                                                         objSettingSize)
                intWindowState = objForm.WindowState
            Case Else                                               ' 通常
                ' 全て退避
                objSettingLocation = objForm.Location
                objSettingSize = objForm.Size
                intWindowState = objForm.WindowState
        End Select
    End Sub

    '***********************************************************************************************
    '   ■■■ ローカルサブプロシージャ ■■■
    '***********************************************************************************************
    '* 処理名　：FP_CheckEnableAdjust
    '* 機能　　：フォーム位置制御可能か判定
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：処理可能判定(Boolean)
    '* 引数　　：Arg1 = 該当フォーム(Object)
    '* 　　　　　Arg2 = 設定退避位置(Point)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年07月07日
    '* 作成者　：井上　治
    '* 更新日　：2016年07月07日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_CheckEnableAdjust(ByRef objForm As Form, _
                                          ByVal objSettingLocation As Point) As Boolean
        '-------------------------------------------------------------------------------------------
        ' 退避位置未設定、フォーム起動位置手動以外は除外
        If ((objSettingLocation.X = 0) AndAlso _
            (objSettingLocation.Y = 0) AndAlso _
            (objForm.StartPosition <> FormStartPosition.Manual)) Then
            Return False
        End If
        ' フォーム起動位置手動以外は｢手動｣にする
        If objForm.StartPosition <> FormStartPosition.Manual Then
            objForm.StartPosition = FormStartPosition.Manual
        End If
        Return True
    End Function

    '***********************************************************************************************
    '* 処理名　：FP_CheckFormInScreens
    '* 機能　　：指定位置・サイズがどれかのスクリーンに収まっているかの判定
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：正常判定(Boolean)
    '* 引数　　：Arg1 = 指定位置(Point)
    '* 　　　　　Arg2 = 指定サイズ(Size)
    '* 　　　　　Arg3 = メインスクリーン(Screen)            ※Option
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年07月07日
    '* 作成者　：井上　治
    '* 更新日　：2016年07月10日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_CheckFormInScreens(ByVal objLocation As Point, _
                                           ByVal objSize As Size, _
                                           Optional ByRef objMainScreen As Screen = Nothing) As Boolean
        '-------------------------------------------------------------------------------------------
        ' 位置未設定は失敗
        If ((objLocation.X = 0) AndAlso (objLocation.Y = 0)) Then Return False
        '-------------------------------------------------------------------------------------------
        Dim objBottomRight As Point = New Point(objLocation.X + objSize.Width, _
                                                objLocation.Y + objSize.Height) ' 右下位置
        Dim blnOK As Boolean = False                                ' 設定スクリーン内判定
        '-------------------------------------------------------------------------------------------
        ' 各スクリーンを巡回してどれかのスクリーン内に収まっているかを確認
        For Each objScreen As Screen In Screen.AllScreens
            ' メインスクリーンは別変数に退避
            If objScreen.Primary Then objMainScreen = objScreen
            With objScreen.WorkingArea
                ' フォームがスクリーン内に収まっているか
                If ((objLocation.X >= .Location.X - g_cnsArrowPoint) AndAlso _
                    (objBottomRight.X <= (.Location.X + .Width + g_cnsArrowPoint)) AndAlso _
                    (objLocation.Y >= .Location.Y - g_cnsArrowPoint) AndAlso _
                    (objBottomRight.Y <= (.Location.Y + .Height + g_cnsArrowPoint))) Then
                    blnOK = True
                End If
            End With
        Next objScreen
        Return blnOK
    End Function

    '***********************************************************************************************
    '* 処理名　：FP_GetPositionDummy
    '* 機能　　：代替フォーム位置の取得(最大化時)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：フォーム位置(Point)
    '* 引数　　：Arg1 = 該当フォーム(Object)
    '* 　　　　　Arg2 = 設定退避位置(Point)
    '* 　　　　　Arg3 = 設定退避サイズ(Size)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年07月07日
    '* 作成者　：井上　治
    '* 更新日　：2016年07月07日
    '* 更新者　：井上　治
    '* 機能説明：フォームが収容されているスクリーンの中央の位置を返す
    '* 注意事項：元の設定値と同じスクリーンの場合は元の設定値を返す
    '***********************************************************************************************
    Private Function FP_GetPositionDummy(ByRef objForm As Form, _
                                         ByVal objSettingLocation As Point, _
                                         ByVal objSettingSize As Size) As Point
        '-------------------------------------------------------------------------------------------
        Dim objLoc1 As Point = objForm.Location                     ' 現在位置
        Dim objLoc2 As Point = objSettingLocation                   ' 退避位置
        Dim strScr1 As String = String.Empty                        ' 現在スクリーン名
        Dim strScr2 As String = String.Empty                        ' 退避スクリーン名
        Dim objCurrScreen As Screen = Nothing                       ' 現在スクリーン
        '-------------------------------------------------------------------------------------------
        ' 最大化(8P程度マイナスになる)のため誤認を避けるため100Pシフトさせておく
        objLoc1.X += 100
        objLoc1.Y += 100
        ' 各スクリーンを巡回してどれかのスクリーン内に収まっているかを確認
        For Each objScreen As Screen In Screen.AllScreens
            With objScreen.WorkingArea
                ' 現在位置が所属しているか
                If ((objLoc1.X >= .Location.X) AndAlso _
                    (objLoc1.X < (.Location.X + .Width)) AndAlso _
                    (objLoc1.Y >= .Location.Y) AndAlso _
                    (objLoc1.Y < (.Location.Y + .Height))) Then
                    objCurrScreen = objScreen
                    strScr1 = objScreen.DeviceName
                End If
                ' 退避位置が所属しているか
                If ((objLoc2.X >= .Location.X) AndAlso _
                    (objLoc2.X < (.Location.X + .Width)) AndAlso _
                    (objLoc2.Y >= .Location.Y) AndAlso _
                    (objLoc2.Y < (.Location.Y + .Height))) Then
                    strScr2 = objScreen.DeviceName
                End If
            End With
        Next objScreen
        '-------------------------------------------------------------------------------------------
        ' 現在スクリーンと退避スクリーンが同じなら退避値のままとする
        If strScr1 = strScr2 Then
            Return objLoc2
        End If
        '-------------------------------------------------------------------------------------------
        ' 退避サイズがゼロの場合はデザインサイズで充当
        If ((objSettingSize.Width = 0) OrElse (objSettingSize.Height = 0)) Then
            objSettingSize = g_objDesigndSize
        End If
        ' 現在スクリーン上の中央に退避サイズのウィンドウがあるとしてその位置を算出
        With objCurrScreen.WorkingArea
            objLoc2 = New Point(.Location.X + (.Width - objSettingSize.Width) \ 2, _
                                .Location.Y + (.Height - objSettingSize.Height) \ 2)
        End With
        Return objLoc2
    End Function

    '---------------------------------------<< End of Source >>-------------------------------------
End Module
