﻿'***************************************************************************************************
'   システム全般の共通定数・変数                        modCommonModule1(Module)
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev     変更履歴内容-------------------------------------------------------------------->
' 11/02/02(1.0.0.0)新規作成
' 12/02/29(1.1.0.0)DPI設定を標準(96DPI)以外にした場合の対応を追加(フォントサイズ可変)
' 13/01/23(1.1.0.1)Debug時のみFP_GET_MyDocumentsPathを追加
' 13/02/26(1.1.0.1)FP_GetFormLocationの追加(複数スクリーン対応)
' 13/07/06(1.1.0.1)NumericUpDownコントロールの対応を追加
' 13/09/04(1.1.0.2)FP_GetFormLocationでのフォームの微弱はみ出し対応
' 14/06/08(1.2.0.0)FP_GET_MyDocumentsPath､FP_GET_PathWithYenSignの廃止(Path.Combineへ移行)
' 14/07/07(1.2.0.0)TextBox等のEnterイベントで文字列を全選択状態にする対応を追加
' 14/08/28(1.2.1.0)DPI関連を｢modFormDpiMgr2｣に移行するため関連記述を削除
' 15/02/14(1.2.1.0)FP_GetTextFromFieldを追加
' 15/10/30(1.2.1.0)GP_SetProgressBarValueを追加
' 16/07/07(1.2.3.0)FP_GetFormLocationを削除(modAdjustLocationSizeに委譲)
'***************************************************************************************************
Imports System.IO
Module modCommonModule1
    '===============================================================================================
    ' 日付チェック用定数
    Friend Const g_cnsDefaultDate As Date = #12/31/1899#
    Friend Const g_cnsMinimumDate As Date = #1/1/1900#
    Friend Const g_cnsMaximumDate As Date = #12/31/2049#
    ' 日付NULL時の代替値
    Friend Const g_cnsNullDate As Date = Nothing
    '-----------------------------------------------------------------------------------------------
    ' システムカラー代替BackColorに関する定数(各Windowsバージョンでの統一)
    Friend ReadOnly g_ColorControl As Color = Color.FromArgb(240, 240, 240)      ' フォームのベース色
    Friend ReadOnly g_ColorControlLight As Color = Color.FromArgb(245, 245, 245) ' 読み取り専用TextBox
    Friend ReadOnly g_ColorMenuBase As Color = Color.FromArgb(183, 208, 248)     ' MenuStrip
    Friend ReadOnly g_ColorStatusBase As Color = Color.FromArgb(225, 225, 225)   ' StatusStrip
    Friend ReadOnly g_ColorStartButton As Color = Color.FromArgb(230, 230, 250)  ' 起動ボタン
    '-----------------------------------------------------------------------------------------------
    ' I/Oエラー時のエラーメッセージ戻り先
    Friend g_strFatalErrMSG As String = ""              ' I/Oエラーメッセージ
    '-----------------------------------------------------------------------------------------------
    ' オーナーフォーム(メッセージ表示用)
    Friend g_objOwnerForm As Object = Nothing

    '***********************************************************************************************
    '   ■■■ 共通サブ処理 ■■■
    ''***********************************************************************************************
    ''* 処理名　：FP_GetFormLocation
    ''* 機能　　：フォームがスクリーン内にあるかの確認
    ''-----------------------------------------------------------------------------------------------
    ''* 返り値　：フォーム位置(Point)
    ''* 引数　　：Arg1 = 保存されていたフォーム位置(Point)
    ''-----------------------------------------------------------------------------------------------
    ''* 作成日　：2013年02月26日
    ''* 作成者　：井上　治
    ''* 更新日　：2013年09月04日
    ''* 更新者　：井上　治
    ''* 機能説明：指定位置がスクリーン上にない場合はデフォルトを返す
    ''* 注意事項：複数スクリーン対応(どれかのスクリーンに位置するかを判定)
    ''***********************************************************************************************
    'Friend Function FP_GetFormLocation(ByVal SavedLocation As Point) As Point
    '    '-------------------------------------------------------------------------------------------
    '    ' 指定位置の設定(ここではフォーム左上端から200px右、20px下としている)
    '    Dim TargetLocation As Point = New Point(SavedLocation.X + 200, _
    '                                            SavedLocation.Y + 20)
    '    '-------------------------------------------------------------------------------------------
    '    ' 存在する全スクリーンを順次探索
    '    Dim blnHit As Boolean = False
    '    For Each objScreen As Screen In Screen.AllScreens
    '        ' 指定位置がスクリーン内にあるかを確認
    '        With objScreen.WorkingArea
    '            If ((TargetLocation.X >= .Left) AndAlso _
    '                (TargetLocation.X < .Left + .Width) AndAlso _
    '                (TargetLocation.Y >= .Top) AndAlso _
    '                (TargetLocation.Y < .Top + .Height)) Then
    '                blnHit = True
    '                Exit For
    '            End If
    '        End With
    '    Next objScreen
    '    '-------------------------------------------------------------------------------------------
    '    ' 現在存在するスクリーン内に位置がなければデフォルトを返す
    '    If blnHit Then
    '        Return SavedLocation
    '    Else
    '        Return New Point(50, 50)
    '    End If
    'End Function

    '***********************************************************************************************
    '* 処理名　：GP_SetProgressBarValue
    '* 機能　　：プログレスバー表示
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 対象プログレスバー(Object)
    '* 　　　　　Arg2 = Value値(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2015年10月30日
    '* 作成者　：井上　治
    '* 更新日　：2015年10月30日
    '* 更新者　：井上　治
    '* 機能説明：Vista以降でもプログレスバーの進捗表示が遅れないように表示する
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_SetProgressBarValue(ByRef objProgressBar As ProgressBar, _
                                      ByVal intValue As Integer)
        '-------------------------------------------------------------------------------------------
        With objProgressBar
            ' 値増加か
            If intValue > .Value Then
                ' 値が増える時
                If intValue < .Maximum Then
                    ' 目的値+1させてから戻す
                    .Value = intValue + 1
                    .Value = intValue
                ElseIf .Value < .Maximum Then
                    ' 最大値を調整
                    .Maximum += 1
                    .Value = .Maximum
                    .Value -= 1
                    .Maximum -= 1
                End If
            Else
                ' 値が減る時はそのまま
                .Value = intValue
            End If
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_AppendMessage
    '* 機能　　：エラーメッセージ累積処理
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 表示用累積メッセージ(String)
    '* 　　　　　Arg2 = 今回追加メッセージ(String)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2009年10月18日
    '* 作成者　：井上　治
    '* 更新日　：2009年10月18日
    '* 更新者　：井上　治
    '* 機能説明：改行を加えながらメッセージを追加する
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_AppendMessage(ByRef strMSG As String, ByRef strADDMSG As String)
        '-------------------------------------------------------------------------------------------
        If strMSG.Length <> 0 Then strMSG &= ControlChars.CrLf
        strMSG &= strADDMSG
    End Sub

    '***********************************************************************************************
    '* 処理名　：FP_GetTextFromField
    '* 機能　　：フィールド(テキスト)の取り出し
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：フィールド値文字列(String)
    '* 引数　　：Arg1 = フィールド(Object)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2011年06月07日
    '* 作成者　：井上　治
    '* 更新日　：2011年06月07日
    '* 更新者　：井上　治
    '* 機能説明：Trim処理及びEmpty対策含む
    '* 注意事項：単一フィールドであること
    '***********************************************************************************************
    Friend Function FP_GetTextFromField(ByRef objFld As Object) As String
        '-------------------------------------------------------------------------------------------
        Return (String.Empty & objFld).ToString.Trim
    End Function

    '---------------------------------------<< End of Source >>-------------------------------------
End Module

