﻿'***************************************************************************************************
'   DataGridView用共通モジュール                        modDataGrigViewTools(Module)
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev     変更履歴内容-------------------------------------------------------------------->
' 14/12/07(1.0.0.0)新規作成
' 16/11/18(1.0.1.0)追加行へのスクロール移動(ScrollToAddedRow)の追加
'***************************************************************************************************
Module modDataGrigViewTools
    '-----------------------------------------------------------------------------------------------
    ' DataGridViewの位置
    Friend Const g_cnsDG_ML As DataGridViewContentAlignment = DataGridViewContentAlignment.MiddleLeft
    Friend Const g_cnsDG_MC As DataGridViewContentAlignment = DataGridViewContentAlignment.MiddleCenter
    Friend Const g_cnsDG_MR As DataGridViewContentAlignment = DataGridViewContentAlignment.MiddleRight
    ' DataGridViewのSortOrder
    Private Const g_cnsSO_Ascending As Windows.Forms.SortOrder = Windows.Forms.SortOrder.Ascending
    Private Const g_cnsSO_Descending As Windows.Forms.SortOrder = Windows.Forms.SortOrder.Descending
    Private Const g_cnsSO_None As Windows.Forms.SortOrder = Windows.Forms.SortOrder.None
    '-----------------------------------------------------------------------------------------------
    ' DataGridViewのカラム設定用ユーザー定義
    Friend Structure g_typDGVColInfo
        Dim dgvCaption As String                        ' 見出し表示名
        Dim dgvWidth As Integer                         ' カラム表示幅
        Dim dgvAlign As DataGridViewContentAlignment    ' 水平配置
        Dim dgvSortMode As DataGridViewColumnSortMode   ' Sortモード
        Dim dgvToolTip As String                        ' 見出しToolTipテキスト
        Dim dgvReadOnly As Boolean                      ' 読み取り専用カラム
    End Structure

    '***********************************************************************************************
    '   ■■■ DataGridView関連共通サブ処理 ■■■
    '***********************************************************************************************
    '* 処理名　：GP_SetColumnInfo
    '* 機能　　：DataGridViewのカラム情報テーブル登録
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = カラム情報テーブル(Array:g_typDGVColInfo)
    '*　　　　　 Arg2 = 見出し表示文字列(String)
    '*　　　　　 Arg3 = カラム表示幅(Integer)
    '*　　　　　 Arg4 = 水平配置(DataGridViewContentAlignment)   ※Option
    '*　　　　　 Arg5 = Sortモード(DataGridViewColumnSortMode)   ※Option
    '*　　　　　 Arg6 = 見出しToolTipテキスト(String)            ※Option
    '*　　　　　 Arg7 = 読み取り専用(Boolean)                    ※Option
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：カラム情報の見出し名､表示幅､配置､Sortモードをカラム順にセット
    '* 注意事項：INDEXは順次加算されるのでテーブルはデフォルトをNothingとする
    '***********************************************************************************************
    Friend Sub GP_SetColumnInfo(ByRef DGVColInfo() As g_typDGVColInfo, _
                                ByVal Caption As String, _
                                ByVal Width As Integer, _
                                Optional ByVal Align As DataGridViewContentAlignment = g_cnsDG_ML, _
                                Optional ByVal SortMode As DataGridViewColumnSortMode = _
                                    DataGridViewColumnSortMode.Automatic, _
                                Optional ByVal ToolTip As String = "", _
                                Optional ByVal blnReadOnly As Boolean = True)
        '-------------------------------------------------------------------------------------------
        ' テーブル要素を追加
        Dim IX As Integer = 0
        If DGVColInfo IsNot Nothing Then
            IX = DGVColInfo.Length
            ReDim Preserve DGVColInfo(IX)
        Else
            ReDim DGVColInfo(IX)
        End If
        '-------------------------------------------------------------------------------------------
        ' テーブルに追加
        With DGVColInfo(IX)
            .dgvCaption = Caption           ' 見出し表示名
            .dgvWidth = Width               ' カラム表示幅
            .dgvAlign = Align               ' 水平配置
            .dgvSortMode = SortMode         ' Sortモード
            .dgvToolTip = ToolTip           ' 見出しToolTipテキスト
            .dgvReadOnly = blnReadOnly      ' 読み取り専用
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：FP_GetDefaultColumnWidth
    '* 機能　　：カラム情報テーブルよりカラム幅一次テーブルを作成
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：カラム幅一次テーブル(Array:Integer)
    '* 引数　　：Arg1 = カラム情報テーブル(Array:g_typDGVColInfo)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：｢列リセット｣用カラム幅一次テーブルを作成する
    '* 注意事項：
    '***********************************************************************************************
    Friend Function FP_GetDefaultColumnWidth(ByRef DGVColInfo() As g_typDGVColInfo) As Integer()
        '-------------------------------------------------------------------------------------------
        Dim IX As Integer = 0, IXMAX As Integer = DGVColInfo.GetUpperBound(0)
        Dim tblDefaultColumnWidth(IXMAX) As Integer
        Do While IX <= IXMAX
            tblDefaultColumnWidth(IX) = DGVColInfo(IX).dgvWidth
            IX += 1
        Loop
        Return tblDefaultColumnWidth.Clone
    End Function

    '***********************************************************************************************
    '* 処理名　：GP_AdjustColumnWidth
    '* 機能　　：カラム情報テーブルのカラム幅再調整
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = カラム情報テーブル(Array:g_typDGVColInfo)
    '*　　　　　 Arg2 = 設定文字列(String)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：列幅の配列を設定退避値で置き換える
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_AdjustColumnWidth(ByRef DGVColInfo() As g_typDGVColInfo, _
                                    ByVal strSettingString As String)
        '-------------------------------------------------------------------------------------------
        If ((strSettingString IsNot Nothing) AndAlso (strSettingString.Length <> 0)) Then
            ' 設定文字列を配列に変換(セパレータはTab)
            Dim tblWidth2 As String() = strSettingString.Split(ControlChars.Tab)
            ' 配列要素の一致を確認
            If ((DGVColInfo.Length > 2) AndAlso (tblWidth2.Length > 2)) Then
                ' 設定値で配列を置き換える
                Dim IX As Integer = 0, IXMAX As Integer = DGVColInfo.GetUpperBound(0)
                If tblWidth2.GetUpperBound(0) < IXMAX Then
                    IXMAX = tblWidth2.GetUpperBound(0)
                End If
                Do While IX <= IXMAX
                    DGVColInfo(IX).dgvWidth = Integer.Parse(tblWidth2(IX))
                    IX += 1
                Loop
            End If
        End If
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_InitDataGridView1
    '* 機能　　：DataGridViewの初期設定(一般一覧用)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = DataGridView(Object)
    '*　　　　　 Arg2 = カラム情報テーブル(Array:g_typDGVColInfo)
    '*　　　　　 Arg3 = 固定見出し列INDEX(Integer)       ※Option(デフォルト:-1)
    '*　　　　　 Arg4 = 見出し高さ(Integer)              ※Option(デフォルト:20)
    '*　　　　　 Arg5 = 選択モード(DGVSelectionMode)     ※Option(デフォルト:FullRow)
    '*　　　　　 Arg6 = 読み取り専用(Boolean)            ※Option(デフォルト:True)
    '*　　　　　 Arg7 = 列幅変更可否(Boolean)            ※Option(デフォルト:True)
    '*　　　　　 Arg8 = 列位置変更可否(Boolean)          ※Option(デフォルト:True)
    '*　　　　　 Arg9 = 先頭列チェックボックス(Boolean)  ※Option(デフォルト:False)
    '*　　　　　 Arg10= 複数選択可否(Boolean)            ※Option(デフォルト:False)
    '*　　　　　 Arg11= 見出し背景色(Color)              ※Option(デフォルト:Form色)
    '*　　　　　 Arg12= 見出し文字色(Color)              ※Option(デフォルト:緑)
    '*　　　　　 Arg13= 行見出し表示(Boolean)            ※Option(デフォルト:False)
    '*　　　　　 Arg14= 列見出し表示(Boolean)            ※Option(デフォルト:True)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_InitDataGridView1(ByRef objDGV As DataGridView, _
                                    ByRef DGVColInfo() As g_typDGVColInfo, _
                                    Optional ByVal FrozenColIndex As Integer = -1, _
                                    Optional ByVal ColumnHeadersHeight As Integer = 20, _
                                    Optional ByVal SelectionMode As DataGridViewSelectionMode = _
                                        DataGridViewSelectionMode.FullRowSelect, _
                                    Optional ByVal blnReadOnly As Boolean = True, _
                                    Optional ByVal AllowUserToResizeColumns As Boolean = True, _
                                    Optional ByVal AllowUserToOrderColumns As Boolean = True, _
                                    Optional ByVal UseCheckBox As Boolean = False, _
                                    Optional ByVal MultiSelect As Boolean = False, _
                                    Optional ByVal ColumnHeaderBackColor As Object = Nothing, _
                                    Optional ByVal ColumnHeaderForeColor As Object = Nothing, _
                                    Optional ByVal RowHeadersVisible As Boolean = False, _
                                    Optional ByVal ColumnHeadersVisible As Boolean = True)
        '-------------------------------------------------------------------------------------------
        Dim IX As Integer = 0, IXMAX As Integer = DGVColInfo.GetUpperBound(0)
        '-------------------------------------------------------------------------------------------
        ' ForeColor、BackColorの調整
        Dim objBackColor As Color = g_ColorControl, objForeColor As Color = Color.Green
        If ColumnHeaderBackColor IsNot Nothing Then
            objBackColor = ColumnHeaderBackColor
        End If
        If ColumnHeaderForeColor IsNot Nothing Then
            objForeColor = ColumnHeaderForeColor
        End If
        '-------------------------------------------------------------------------------------------
        ' DataGridViewの初期設定
        With objDGV
            .AutoSize = False                                       ' 自動サイズ無し(固定)
            .AllowUserToResizeColumns = AllowUserToResizeColumns    ' 列幅変更可否
            .AllowUserToOrderColumns = AllowUserToOrderColumns      ' 列位置変更可否
            .AllowUserToResizeRows = False                          ' 行高変更不可(固定)
            .EnableHeadersVisualStyles = False                      ' Visualスタイルを使用しない(固定)
            .RowHeadersVisible = RowHeadersVisible                  ' 行見出し有無
            If RowHeadersVisible Then
                With .RowHeadersDefaultCellStyle
                    .Alignment = DataGridViewContentAlignment.MiddleCenter  ' (固定)
                    .BackColor = objBackColor                       ' 列見出し背景色
                    .ForeColor = objForeColor                       ' 列見出し文字色
                End With
            End If
            .ColumnHeadersVisible = ColumnHeadersVisible            ' 列見出し有無
            .ReadOnly = blnReadOnly                                 ' 読み取り専用
            .ColumnHeadersHeightSizeMode = _
                DataGridViewColumnHeadersHeightSizeMode.DisableResizing ' (固定)
            With .ColumnHeadersDefaultCellStyle
                .Alignment = DataGridViewContentAlignment.MiddleCenter  ' (固定)
                .BackColor = objBackColor                           ' 列見出し背景色
                .ForeColor = objForeColor                           ' 列見出し文字色
            End With
            .ColumnHeadersHeight = ColumnHeadersHeight              ' 列見出し高さ
            .SelectionMode = SelectionMode                          ' 選択モード
            .MultiSelect = MultiSelect                              ' 複数選択
            If UseCheckBox Then
                ' 先頭列にチェックボックスを配置
                Dim objDGV_CBC As New DataGridViewCheckBoxColumn    ' チェックボックス列
                .Columns.Add(objDGV_CBC)
            End If
            .ColumnCount = IXMAX + 1                                ' 列数
            ' 各列の設定
            Do While IX <= IXMAX
                With .Columns(IX)
                    .Width = DGVColInfo(IX).dgvWidth
                    .HeaderText = DGVColInfo(IX).dgvCaption
                    .AutoSizeMode = DataGridViewAutoSizeColumnMode.None
                    .DefaultCellStyle.Alignment = DGVColInfo(IX).dgvAlign
                    .SortMode = DGVColInfo(IX).dgvSortMode
                    If DGVColInfo(IX).dgvToolTip.Length <> 0 Then
                        .ToolTipText = DGVColInfo(IX).dgvToolTip
                    End If
                    If Not blnReadOnly Then
                        .ReadOnly = DGVColInfo(IX).dgvReadOnly
                    End If
                End With
                IX += 1
            Loop
            If FrozenColIndex >= 0 Then
                .Columns(FrozenColIndex).Frozen = True              ' 設定列で列固定
            End If
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：FP_GetDGVColumnWidth
    '* 機能　　：DataGridViewのカラム幅の取得
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：設定退避用に文字列(String)
    '* 引数　　：Arg1 = DataGridView(Object)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：DataGridViewのカラム幅を設定退避用に文字列で取得
    '* 注意事項：DPI制御時は96DPI相当に割り戻す対応を行なっている
    '***********************************************************************************************
    Friend Function FP_GetDGVColumnWidth(ByRef objDGV As DataGridView) As String
        '-------------------------------------------------------------------------------------------
        Dim strTEXT As String = String.Empty
        Dim IX As Integer = 1
        With objDGV
            If g_sngMagnifyingPowerX = 1.0F Then
                strTEXT = .Columns(0).Width.ToString
                Do While IX < .Columns.Count
                    strTEXT &= ControlChars.Tab & .Columns(IX).Width.ToString
                    IX += 1
                Loop
            Else
                strTEXT = CType(Math.Round(.Columns(0).Width / g_sngMagnifyingPowerX), Integer).ToString
                Do While IX < .Columns.Count
                    strTEXT &= ControlChars.Tab & CType(Math.Round(.Columns(IX).Width / g_sngMagnifyingPowerX), Integer).ToString
                    IX += 1
                Loop
            End If
        End With
        Return strTEXT
    End Function

    '***********************************************************************************************
    '* 処理名　：GP_AdjustDGVColumnDisplayIndex
    '* 機能　　：DataGridViewのカラム配置再調整
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = DataGridView(Object)
    '*　　　　　 Arg2 = 設定文字列(String)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：列配置を設定退避値で置き換える
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub GP_AdjustDGVColumnDisplayIndex(ByRef objDGV As DataGridView, _
                                              ByVal strSettingString As String)
        '-------------------------------------------------------------------------------------------
        If ((strSettingString IsNot Nothing) AndAlso (strSettingString.Length <> 0)) Then
            ' 設定文字列を配列に変換(セパレータはTab)
            Dim tblDisplayIndexS As String() = strSettingString.Split(ControlChars.Tab)
            Dim IXMAX As Integer = tblDisplayIndexS.GetUpperBound(0)
            Dim tblDisplayIndex(IXMAX) As Integer, tblIndex(IXMAX) As Integer
            Dim IX As Integer = 0
            Do While IX <= IXMAX
                tblDisplayIndex(IX) = Integer.Parse(tblDisplayIndexS(IX))
                tblIndex(IX) = IX
                IX += 1
            Loop
            ' 表示順の若い方からセットするために並替え
            Array.Sort(tblDisplayIndex, tblIndex)
            With objDGV
                If IXMAX >= .Columns.Count Then
                    IXMAX = .Columns.Count - 1
                End If
                IX = 0
                Do While IX <= IXMAX
                    If tblIndex(IX) <= IXMAX Then
                        .Columns(tblIndex(IX)).DisplayIndex = IX
                    End If
                    IX += 1
                Loop
            End With
        End If
    End Sub

    '***********************************************************************************************
    '* 処理名　：FP_GetDGVColumnDisplayIndex
    '* 機能　　：DataGridViewのカラム配置の取得
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：設定退避用に文字列(String)
    '* 引数　　：Arg1 = DataGridView(Object)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：DataGridViewのカラム配置を設定退避用に文字列で取得
    '* 注意事項：
    '***********************************************************************************************
    Friend Function FP_GetDGVColumnDisplayIndex(ByRef objDGV As DataGridView) As String
        '-------------------------------------------------------------------------------------------
        Dim strTEXT As String = String.Empty
        Dim IX As Integer = 1
        With objDGV
            strTEXT = .Columns(0).DisplayIndex.ToString
            Do While IX < .Columns.Count
                strTEXT &= ControlChars.Tab & .Columns(IX).DisplayIndex.ToString
                IX += 1
            Loop
        End With
        Return strTEXT
    End Function

    '***********************************************************************************************
    '* 処理名　：GP_ResetDGVColumnChange
    '* 機能　　：DataGridViewのカラム幅・配置変更のリセット
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 表示フォーム(Object)
    '*　　　　　 Arg2 = DataGridView(Object)
    '*　　　　　 Arg3 = カラム幅(Array:Integer)
    '*　　　　　 Arg4 = フォームタイトル(String)
    '*　　　　　 Arg5 = 確認メッセージ抑止(Boolean)      ※Option
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：カラム幅・列配置をデフォルトで置き換える
    '* 注意事項：テーブル要素数とDataGridViewの列数が一致していること
    '***********************************************************************************************
    Friend Sub GP_ResetDGVColumnChange(ByRef objForm As Object, _
                                       ByRef objDGV As DataGridView, _
                                       ByVal tblWidth As Integer(), _
                                       ByVal strTitle As String, _
                                       Optional ByVal swOmitCommit As Boolean = False)
        '-------------------------------------------------------------------------------------------
        ' 起動確認
        If Not swOmitCommit Then
            If MessageBox2.DialogShow(objForm, _
                                      "グリッド表示の列幅・配置変更をリセットしますか？", _
                                      strTitle, _
                                      MessageBoxButtons.YesNo, _
                                      MessageBoxIcon.Information) <> DialogResult.Yes Then Exit Sub
        End If
        '-------------------------------------------------------------------------------------------
        Dim IX As Integer = 0, IXMAX As Integer = tblWidth.GetUpperBound(0)
        Dim IXMAX2 As Integer = objDGV.Columns.Count - 1
        If IXMAX2 < IXMAX Then IXMAX = IXMAX2
        With objDGV
            .Visible = False
            Do While IX <= IXMAX
                With .Columns(IX)
                    .Width = tblWidth(IX) * g_sngMagnifyingPowerX
                    .DisplayIndex = IX
                End With
                IX += 1
            Loop
            .Visible = True
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：DGV_ColumnHeaderMouseClick
    '* 機能　　：グリッド見出しクリック(ColumnHeaderMouseClick)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2014年12月07日
    '* 作成者　：井上　治
    '* 更新日　：2014年12月07日
    '* 更新者　：井上　治
    '* 機能説明：SortMode.Programmatic列を含む一覧の並替えを行なう
    '* 注意事項：Programmatic列は実Sort列IndexをTagにセットしておくこと
    '***********************************************************************************************
    Friend Sub DGV_ColumnHeaderMouseClick(ByVal sender As Object, _
                                          ByVal e As DataGridViewCellMouseEventArgs)
        '-------------------------------------------------------------------------------------------
        With CType(sender, DataGridView)
            Dim COL As Integer = e.ColumnIndex
            If .Columns(COL).SortMode = DataGridViewColumnSortMode.Programmatic Then
                '-----------------------------------------------------------------------------------
                ' 当該カラムがProgrammaticに場合は指定列でSort
                Dim COL2 As Integer = .Columns(COL).Tag
                If .Columns(COL).HeaderCell.SortGlyphDirection = g_cnsSO_Ascending Then
                    ' 降順に並替え及びソートマーク切替え
                    .Sort(.Columns(COL2), System.ComponentModel.ListSortDirection.Descending)
                    .Columns(COL).HeaderCell.SortGlyphDirection = g_cnsSO_Descending
                Else
                    ' 昇順に並替え及びソートマーク切替え
                    .Sort(.Columns(COL2), System.ComponentModel.ListSortDirection.Ascending)
                    .Columns(COL).HeaderCell.SortGlyphDirection = g_cnsSO_Ascending
                End If
                ' 他のProgrammatic列のソートマークを消す
                For Each objCol As DataGridViewColumn In .Columns
                    If ((objCol.SortMode = DataGridViewColumnSortMode.Programmatic) AndAlso _
                        (objCol.Index <> COL)) Then
                        objCol.HeaderCell.SortGlyphDirection = g_cnsSO_None
                    End If
                Next objCol
            Else
                '-----------------------------------------------------------------------------------
                ' 当該カラムがProgrammaticでない場合は全Programmatic列のソートマークを消す
                For Each objCol As DataGridViewColumn In .Columns
                    If objCol.SortMode = DataGridViewColumnSortMode.Programmatic Then
                        objCol.HeaderCell.SortGlyphDirection = g_cnsSO_None
                    End If
                Next objCol
            End If
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：ScrollToAddedRow
    '* 機能　　：追加行へのスクロール移動
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = DataGridView(Object)
    '* 　　　　　Arg2 = 追加された行INDEX(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2016年11月18日
    '* 作成者　：井上　治
    '* 更新日　：2016年11月18日
    '* 更新者　：井上　治
    '* 機能説明：新規登録で最終行に行が追加された時にその行にスクロール表示させる
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub ScrollToAddedRow(ByRef objDGV As DataGridView, _
                                ByVal intNewRow As Integer)
        '-------------------------------------------------------------------------------------------
        ' 追加行が画面内の場合は何もしない
        If ((intNewRow < 0) OrElse (objDGV.Rows(intNewRow).Displayed)) Then Exit Sub
        Dim intLastRow As Integer = objDGV.Rows.Count - 1           ' 最終行 
        ' 追加行が最終行でない場合は何もしない
        If intNewRow <> intLastRow Then Exit Sub
        With objDGV
            Dim intCntRow As Integer = 0                            ' 表示可能行数
            Dim intRow As Integer = .FirstDisplayedScrollingRowIndex ' 表示先頭行INDEX
            ' 下へ向かって各行(行数)が表示可能かを確認(可変行高は不対応)
            Do While intRow <= intLastRow
                ' 現在行が表示されているか
                If .Rows(intRow).Displayed Then
                    intCntRow += 1
                Else
                    Exit Do
                End If
                ' 次の行へ
                intRow += 1
            Loop
            ' 表示可能行数が取得できない場合は何もしない
            If intCntRow = 0 Then Exit Sub
            ' 一部表示でもDisplayedになることの対応と最終行認識のため空行を表示させるため2行調整
            intRow = intNewRow - intCntRow + 2
            ' 先頭判定(念のため！)
            If intRow < 0 Then intRow = 0
            ' その行にスクロールさせる
            .FirstDisplayedScrollingRowIndex = intRow
            ' 追加行を選択状態にする
            .Rows(intNewRow).Selected = True
        End With
    End Sub

    '---------------------------------------<< End of Source >>-------------------------------------
End Module
