﻿'***************************************************************************************************
'     祝日パラメータ登録(詳細画面)                               dlgSetHoliParam(Form)
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev     変更履歴内容-------------------------------------------------------------------->
' 18/12/16(1.0.2.0)新規作成
'***************************************************************************************************
Friend Class dlgSetHoliParam
    '===============================================================================================
    ' 共通定数(既定)
    Private Const g_cnsProgTitle As String = "祝日パラメータ登録"
    '-----------------------------------------------------------------------------------------------
    ' 処理区分コンボの№リスト
    Private ReadOnly g_tblSyoriKbn() As Integer = New Integer() {0, 1, 2, 9}
    '-----------------------------------------------------------------------------------------------
    ' ダブルクリック抑制クラス(初期化はNewプロシージャで行なう)
    Private g_objOmitDoubleClick As clsOmitDoubleClick2         ' ダブルクリック抑制クラス
    Private g_blnEnabledODCClass As Boolean = False             ' ダブルクリック抑制クラス利用可
    ' 未登録終了警告スイッチ対応クラス(初期化はNewプロシージャで行なう)
    Private g_clsNotUpdate As clsCheckNotUpdate3                ' 未登録終了警告スイッチ対応クラス
    '-----------------------------------------------------------------------------------------------
    ' 親フォームからの引き渡し情報
    Private g_intUpdateMode As Integer                          ' 処理モード(0=参照,1=新規,2=変更)
    Private g_intUpdateResult As Integer = 0                    ' 本処理の更新結果(0=無,1=追加,2=更新,9=削除)
    Private g_intSeq As Integer                                 ' SEQ
    '-----------------------------------------------------------------------------------------------
    ' カスタムコントロールから利用される背面コントロール(既定⇒modFormDpiMgr1から使用される)
    Public g_tblCustomLabel() As Object = Nothing               ' コントロールテーブル

    '***********************************************************************************************
    ' ■■■ 初期化 ■■■
    '***********************************************************************************************
    '* 処理名　：New
    '* 機能　　：全体初期化
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Friend Sub New()
        '-------------------------------------------------------------------------------------------
        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()
        ' フォームデザイナモード時は以下をスキップする
        If Me.DesignMode Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック抑制クラスの初期化(FormClosingイベント無し)
        g_objOmitDoubleClick = New clsOmitDoubleClick2(Me, False, g_blnEnabledODCClass)
        '-------------------------------------------------------------------------------------------
        ' DPI値によるフォーム／コントロールの位置・サイズ調整(カスタムコントロール調整を含む)
        Call GP_AdjustFormSizeByDpi(Me)
        '-------------------------------------------------------------------------------------------
        ' 未登録終了警告スイッチ対応クラス初期化(イベントハンドラ追加)
        g_clsNotUpdate = New clsCheckNotUpdate3(Me, g_objOmitDoubleClick)
    End Sub

    '***********************************************************************************************
    ' ■■■ フォームイベント ■■■
    '***********************************************************************************************
    '* 処理名　：Form_Load
    '* 機能　　：フォームイベント(Load)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub Form_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        '-------------------------------------------------------------------------------------------
        ' 全面パネルを非表示
        Panel1.Visible = False
        ' メニューを一旦非表示にする
        MNU_NEW.Visible = False
        MNU_COPY.Visible = False
        MNU_DEL.Visible = False
        BTN_OK.Visible = False
        CBO_GETSU.Visible = False
        LBL_SEQ.Visible = False
        LBL_SEQ2.Visible = False
    End Sub

    '***********************************************************************************************
    '* 処理名　：Form_Shown
    '* 機能　　：フォーム表示終段(Form_Shown)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub Form_Shown(ByVal sender As Object, _
                           ByVal e As System.EventArgs) Handles Me.Shown
        '-------------------------------------------------------------------------------------------
        ' 致命例外時のスキップ判定スイッチをクリア
        g_clsNotUpdate.FatalError = False
        g_intUpdateResult = 0
        ' フォーム起動位置モードを手動に変更
        If Me.StartPosition <> FormStartPosition.Manual Then
            Me.StartPosition = FormStartPosition.Manual
        End If
        '-------------------------------------------------------------------------------------------
        ' IME状態の初期化(Disable以外)
        TXT_SYUKU_NM.ImeMode = Windows.Forms.ImeMode.Hiragana
        '-------------------------------------------------------------------------------------------
        ' 新規登録モード
        If g_intUpdateMode = 1 Then
            ' 内容表示クリア
            CBO_GETSU.SelectedIndex = -1
            Call GP_ClearForm()
            ' 処理区分による表示制御
            Call GP_SetSyoriKbn()
        Else
            ' フォーム内容表示
            Call GP_ShowForm()
        End If
        ' フォームの状態制御
        Call GP_SetFormCondition()
        '-------------------------------------------------------------------------------------------
        ' 全面パネルを表示
        Panel1.Visible = True
        Me.Activate()
        Me.Refresh()
        ' 新規登録モード
        If g_intUpdateMode = 1 Then
            CBO_GETSU.Select()
        Else
            CBO_SYORI_KBN.Select()
        End If
        ' 未登録終了警告スイッチをクリア
        g_clsNotUpdate.NotUpdate = False
        ' 初期表示動作完了
        g_objOmitDoubleClick.Shown = True
    End Sub

    '***********************************************************************************************
    ' ■■■ コントロールイベント ■■■
    '***********************************************************************************************
    '* 処理名　：BTN_OK_Click
    '* 機能　　：「登録」ボタンイベント(Click)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub BTN_OK_Click(ByVal sender As Object, _
                             ByVal e As System.EventArgs) Handles BTN_OK.Click
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック等の多重操作を抑制
        If Not g_blnEnabledODCClass OrElse g_objOmitDoubleClick.CheckDoubleClick() Then Exit Sub
        '-------------------------------------------------------------------------------------------
        BTN_OK.Focus()
        Application.DoEvents()
        ' ｢登録｣ボタンClick
        Call GP_OK_Click()
    End Sub

    '***********************************************************************************************
    '* 処理名　：BTN_CANCEL_Click
    '* 機能　　：｢キャンセル｣ボタンイベント(Click)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub BTN_CANCEL_Click(ByVal sender As Object, _
                                 ByVal e As System.EventArgs) Handles BTN_CANCEL.Click
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック等の多重操作を抑制
        If Not g_blnEnabledODCClass OrElse g_objOmitDoubleClick.CheckDoubleClick() Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' フォームを閉じる
        Me.Close()
    End Sub

    '***********************************************************************************************
    '* 処理名　：CCB_SYORI_KBN_SelectedIndexChanged
    '* 機能　　：「処理区分」コンボイベント(SelectedIndexChanged)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub CCB_SYORI_KBN_SelectedIndexChanged(ByVal sender As Object, _
                                                   ByVal e As System.EventArgs) _
                                                   Handles CBO_SYORI_KBN.SelectedIndexChanged
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック等の多重操作を抑制
        If Not g_blnEnabledODCClass OrElse g_objOmitDoubleClick.CheckDoubleClick() Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' 処理区分による表示制御
        Call GP_SetSyoriKbn()
    End Sub

    '***********************************************************************************************
    '* 処理名　：MNU_NEW_Click
    '* 機能　　：「新規登録」メニューイベント(Click)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub MNU_NEW_Click(ByVal sender As Object, _
                              ByVal e As System.EventArgs) Handles MNU_NEW.Click
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック等の多重操作を抑制
        If Not g_blnEnabledODCClass OrElse g_objOmitDoubleClick.CheckDoubleClick() Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' 処理確認
        If MessageBox2.DialogShow( _
            Me, _
            "現在の表示内容を破棄して新規登録画面に切り替えます。" & _
            ControlChars.CrLf & "よろしいですね？", _
            g_cnsProgTitle, _
            MessageBoxButtons.YesNo, _
            MessageBoxIcon.Information) <> Windows.Forms.DialogResult.Yes Then Exit Sub
        ' 処理中判定スイッチ対応
        g_objOmitDoubleClick.OmitDoubleClick = True
        '-------------------------------------------------------------------------------------------
        ' 内容表示クリア
        Call GP_ClearForm()
        g_intUpdateMode = 1                              ' 新規登録に変更
        ' フォームの状態制御
        Call GP_SetFormCondition()
        ' 未登録終了警告スイッチをクリア
        g_clsNotUpdate.NotUpdate = False
        ' 月にフォーカス
        CBO_GETSU.Focus()
    End Sub

    '***********************************************************************************************
    '* 処理名　：MNU_COPY_Click
    '* 機能　　：「コピー登録」メニューイベント(Click)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub MNU_COPY_Click(ByVal sender As Object, _
                               ByVal e As System.EventArgs) Handles MNU_COPY.Click
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック等の多重操作を抑制
        If Not g_blnEnabledODCClass OrElse g_objOmitDoubleClick.CheckDoubleClick() Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' 処理確認
        If MessageBox2.DialogShow( _
            Me, _
            "現在の表示内容を保持して新規登録画面に切り替えます。" & _
            ControlChars.CrLf & "よろしいですね？", _
            g_cnsProgTitle, _
            MessageBoxButtons.YesNo, _
            MessageBoxIcon.Information) <> Windows.Forms.DialogResult.Yes Then Exit Sub
        ' 処理中判定スイッチ対応
        g_objOmitDoubleClick.OmitDoubleClick = True
        '-------------------------------------------------------------------------------------------
        g_intUpdateMode = 1                              ' 新規登録に変更
        ' フォームの状態制御
        Call GP_SetFormCondition()
        ' 未登録終了警告スイッチをクリア
        g_clsNotUpdate.NotUpdate = False
        ' 月にフォーカス
        CBO_GETSU.Focus()
    End Sub

    '***********************************************************************************************
    '* 処理名　：MNU_DEL_Click
    '* 機能　　：「削除」メニューイベント(Click)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub MNU_DEL_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) Handles MNU_DEL.Click
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック等の多重操作を抑制
        If Not g_blnEnabledODCClass OrElse g_objOmitDoubleClick.CheckDoubleClick() Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' ｢削除｣メニューボタンClickサブ
        Call GP_DELETE_Click()
    End Sub

    '***********************************************************************************************
    ' ■■■ サブ処理(登録更新系) ■■■
    '***********************************************************************************************
    '* 処理名　：GP_OK_Click
    '* 機能　　：｢登録｣ボタンClickサブ
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_OK_Click()
        '-------------------------------------------------------------------------------------------
        ' 入力内容チェック
        If Not g_clsNotUpdate.NotUpdate Then
            ' 何も入力／変更がない
            MessageBox2.DialogShow(Me, _
                                   g_cnsMSG051, _
                                   g_cnsProgTitle, _
                                   MessageBoxButtons.OK, _
                                   MessageBoxIcon.Exclamation)
            Exit Sub
        ElseIf Not FP_CheckForm() Then  ' 入力チェック
            Exit Sub
        Else                            ' 処理なし
        End If
        '-------------------------------------------------------------------------------------------
        ' 処理確認
        If MessageBox2.DialogShow( _
            Me, _
            g_cnsProgTitle & "に表示されている内容を登録します。" & _
            ControlChars.CrLf & ControlChars.CrLf & "よろしいですね？", _
            g_cnsProgTitle, _
            MessageBoxButtons.YesNo, _
            MessageBoxIcon.Information) <> Windows.Forms.DialogResult.Yes Then Exit Sub
        ' 処理中判定スイッチ対応
        g_objOmitDoubleClick.OmitDoubleClick = True
        '-------------------------------------------------------------------------------------------
        ' 登録(更新)処理
        Call GP_UpdateTables()
        ' 処理結果を返す
        g_intUpdateResult = g_intUpdateMode
        ' 未登録終了警告スイッチをクリア
        g_clsNotUpdate.NotUpdate = False
        ' 閉じる
        Me.Close()
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_DELETE_Click
    '* 機能　　：｢削除｣メニューボタンClickサブ
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_DELETE_Click()
        '-------------------------------------------------------------------------------------------
        ' 処理確認
        If MessageBox2.DialogShow( _
            Me, _
            g_cnsProgTitle & "に表示されている内容を削除します。" & _
            ControlChars.CrLf & "よろしいですね？", _
            g_cnsProgTitle, _
            MessageBoxButtons.YesNo, _
            MessageBoxIcon.Information) <> Windows.Forms.DialogResult.Yes Then Exit Sub
        ' 処理中判定スイッチ対応
        g_objOmitDoubleClick.OmitDoubleClick = True
        '-------------------------------------------------------------------------------------------
        g_intUpdateMode = 9      ' 削除
        ' 祝日情報の削除
        Call GP_RemoveHoliMember(g_intSeq)
        ' 処理結果を返す
        g_intUpdateResult = g_intUpdateMode
        ' 未登録終了警告スイッチをクリア
        g_clsNotUpdate.NotUpdate = False
        ' 閉じる
        Me.Close()
    End Sub

    '***********************************************************************************************
    '* 処理名　：FP_CheckForm
    '* 機能　　：フォーム登録内容チェック
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：処理正否(Boolean)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_CheckForm() As Boolean
        '-------------------------------------------------------------------------------------------
        Dim strMSG As String = String.Empty                         ' メッセージ
        If g_intUpdateMode = 1 Then    ' 新規登録時のみ
            ' 月が選択されているか
            If CBO_GETSU.SelectedIndex < 0 Then
                Call GP_AppendMessage(strMSG, "「月" & g_cnsERRMSG02)
            End If
        End If
        Dim intSyoriKbn As Integer = FP_GetSyoriKbnCombo()          ' 処理区分
        ' 処理区分が選択されているか
        If intSyoriKbn < 0 Then
            Call GP_AppendMessage(strMSG, "「処理区分" & g_cnsERRMSG02)
        ElseIf ((intSyoriKbn = 0) OrElse (intSyoriKbn = 2)) Then
            ' 固定日
            If NUD_HIDUKE.Value = 0 Then
                Call GP_AppendMessage(strMSG, "「日(固定日)" & g_cnsERRMSG01)
            End If
            ' 振替区分
            If CBO_FURIKAE_KBN.SelectedIndex < 0 Then
                Call GP_AppendMessage(strMSG, "「振替区分(固定日)" & g_cnsERRMSG02)
            End If
        ElseIf intSyoriKbn = 1 Then
            ' HappyMonday
            If NUD_SYUSU.Value = 0 Then
                Call GP_AppendMessage(strMSG, "「週数(HappyMonday)" & g_cnsERRMSG01)
            End If
            ' 曜日
            If CBO_YOBI.SelectedIndex <> 1 Then
                Call GP_AppendMessage(strMSG, "「曜日(HappyMonday)" & g_cnsERRMSG02)
            End If
        End If
        ' 祝日名が入力されているか
        If TXT_SYUKU_NM.Text.Trim.Length = 0 Then
            Call GP_AppendMessage(strMSG, "「祝日名" & g_cnsERRMSG01)
        End If
        ' 終了年が入力されているか
        If NUD_END_YEAR.Value = 0 Then
            Call GP_AppendMessage(strMSG, "「終了年" & g_cnsERRMSG01)
        ElseIf NUD_END_YEAR.Value < NUD_STR_YEAR.Value Then
            Call GP_AppendMessage(strMSG, "「有効年期間" & g_cnsERRMSG41)
        End If
        ' ここまででエラーがあれば終了
        If strMSG.Length <> 0 Then
            MessageBox2.DialogShow(Me, _
                                   strMSG, _
                                   g_cnsProgTitle, _
                                   MessageBoxButtons.OK, _
                                   MessageBoxIcon.Exclamation)
            Return False
        End If
        '-------------------------------------------------------------------------------------------
        Dim intIxMax As Integer = g_tblHoliParamater.GetUpperBound(0) ' テーブルINDEX
        Dim intIx As Integer = 0                                    ' テーブルINDEX
        Dim intGetsu As Integer = CBO_GETSU.SelectedIndex + 1       ' 月 
        Dim intStrYear As Integer = NUD_STR_YEAR.Value              ' 開始年 
        Dim intEndYear As Integer = NUD_END_YEAR.Value              ' 終了年 
        ' 同月で同日(同週)か同名の他のレコードと有効年期間が重複しないか判定
        Do While intIx <= intIxMax
            ' 他IDか
            If intIx <> g_intSeq Then
                Dim intGetsu2 As Integer = g_tblHoliParamater(intIx).Getsu ' 月 
                Dim intStrYear2 As Integer = g_tblHoliParamater(intIx).StrYear ' 開始年
                Dim intEndYear2 As Integer = g_tblHoliParamater(intIx).EndYear ' 開始年
                ' 同一月で有効年期間に重複があるか
                If ((intGetsu2 = intGetsu) AndAlso _
                    (((intStrYear >= intStrYear2) AndAlso (intStrYear <= intEndYear2)) OrElse _
                     ((intEndYear >= intStrYear2) AndAlso (intEndYear <= intEndYear2)) OrElse _
                     ((intStrYear2 >= intStrYear) AndAlso (intStrYear2 <= intEndYear)) OrElse _
                     ((intEndYear2 >= intStrYear) AndAlso (intEndYear2 <= intEndYear)))) Then
                    ' 区分、日付等の重複判定
                    If Not FP_CheckDetail(intIx) Then
                        Call GP_AppendMessage(strMSG, "「有効年期間」が他と重複しています。")
                        Exit Do
                    End If
                End If
            End If
            ' 次へ
            intIx += 1
        Loop
        '-------------------------------------------------------------------------------------------
        ' チェック結果判定
        If strMSG.Length <> 0 Then
            MessageBox2.DialogShow(Me, _
                                   strMSG, _
                                   g_cnsProgTitle, _
                                   MessageBoxButtons.OK, _
                                   MessageBoxIcon.Exclamation)
            Return False
        Else                ' 成功
            Return True
        End If
    End Function

    '***********************************************************************************************
    '* 処理名　：FP_CheckDetail
    '* 機能　　：区分、日付等の重複判定
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：チェック成否(Boolean)
    '* 引数　　：Arg1 = テーブルINDEX(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：現在INDEXの登録内容と各区分等の一致を見る(月、年期間は上位で判定済)
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_CheckDetail(ByVal intIx As Integer) As Boolean
        '-------------------------------------------------------------------------------------------
        With g_tblHoliParamater(intIx)
            ' 同じ処理区分でなければ許可
            If .SyoriKbn <> g_tblSyoriKbn(CBO_SYORI_KBN.SelectedIndex) Then Return True
            ' 処理区分
            If ((.SyoriKbn = 0) OrElse (.SyoriKbn = 2)) Then
                Return NUD_HIDUKE.Value <> .Hiduke
            ElseIf .SyoriKbn = 1 Then
                Return NUD_SYUSU.Value <> .HmSyusu
            Else
                Return False
            End If
        End With
    End Function

    '***********************************************************************************************
    '* 処理名　：GP_UpdateTables
    '* 機能　　：登録(更新)処理
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_UpdateTables()
        '-------------------------------------------------------------------------------------------
        ' 新規登録時はテーブル要素を追加
        If g_intUpdateMode = 1 Then
            g_intSeq = g_tblHoliParamater.Length
            ReDim Preserve g_tblHoliParamater(g_intSeq)
        End If
        ' 祝日情報の登録(単一要素)
        Call GP_SetHoliPara(g_intSeq)
        ' 祝日情報の並替え
        Call GP_SortHoliPara()
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_SetHoliPara
    '* 機能　　：祝日情報の登録(単一要素)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = テーブルINDEX(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_SetHoliPara(ByVal intIx As Integer)
        '-------------------------------------------------------------------------------------------
        With g_tblHoliParamater(intIx)
            .Getsu = CBO_GETSU.SelectedIndex + 1
            .SyoriKbn = g_tblSyoriKbn(CBO_SYORI_KBN.SelectedIndex)
            ' 処理区分判定
            If ((.SyoriKbn = 0) OrElse (.SyoriKbn = 2)) Then
                .Hiduke = NUD_HIDUKE.Value
                .FurikaeKbn = CBO_FURIKAE_KBN.SelectedIndex
                .HmSyusu = 0
                .HmYobi = 0
            ElseIf .SyoriKbn = 1 Then
                .Hiduke = 0
                .FurikaeKbn = 0
                .HmSyusu = NUD_SYUSU.Value
                .HmYobi = CBO_YOBI.SelectedIndex
            Else
                .Hiduke = 0
                .FurikaeKbn = 0
                .HmSyusu = 0
                .HmYobi = 0
            End If
            .SyukuNm = TXT_SYUKU_NM.Text.ToString.Trim
            .StrYear = NUD_STR_YEAR.Value
            .EndYear = NUD_END_YEAR.Value
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_SortHoliPara
    '* 機能　　：祝日情報の並替え
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：月日順に並び替える
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_SortHoliPara()
        '-------------------------------------------------------------------------------------------
        Dim intIxMax As Integer = g_tblHoliParamater.GetUpperBound(0) ' テーブルINDEX
        Dim intIx As Integer = 0                                    ' テーブルINDEX
        Dim tblKey(intIxMax) As String                              ' SORTキーテーブル
        '-------------------------------------------------------------------------------------------
        ' キーテーブルを作成
        Do While intIx <= intIxMax
            With g_tblHoliParamater(intIx)
                tblKey(intIx) = .Getsu.ToString("00")               ' 月
                ' 処理区分判定
                Select Case .SyoriKbn
                    Case 0, 2                               ' 通常
                        tblKey(intIx) &= .Hiduke.ToString("00")
                    Case 1                                  ' HappyMonday
                        tblKey(intIx) &= (.HmSyusu * 7).ToString("00")
                    Case Else                               ' 特殊
                        tblKey(intIx) &= "23"
                End Select
                tblKey(intIx) &= .StrYear.ToString                  ' 開始年
            End With
            intIx += 1
        Loop
        '-------------------------------------------------------------------------------------------
        ' 並び替え
        Array.Sort(tblKey, g_tblHoliParamater)
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_RemoveHoliMember
    '* 機能　　：祝日情報の削除
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 削除位置INDEX(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_RemoveHoliMember(ByVal intIx As Integer)
        '-------------------------------------------------------------------------------------------
        Dim intIxMax As Integer = g_tblHoliParamater.GetUpperBound(0) ' テーブルINDEX
        ' 削除する要素＋１～の内容 → 削除する要素～にコピー
        Array.Copy(g_tblHoliParamater, intIx + 1, g_tblHoliParamater, intIx, intIxMax - intIx)
        ' 最終行を削除する
        ReDim Preserve g_tblHoliParamater(intIxMax - 1)
    End Sub

    '***********************************************************************************************
    ' ■■■ サブ処理(表示系) ■■■
    '***********************************************************************************************
    '* 処理名　：GP_ShowForm
    '* 機能　　：フォーム内容表示
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_ShowForm()
        '-------------------------------------------------------------------------------------------
        ' 内容表示クリア
        Call GP_ClearForm()
        ' 処理中判定スイッチ対応
        g_objOmitDoubleClick.OmitDoubleClick = True
        '-------------------------------------------------------------------------------------------
        ' レコード内容をコントロールにセット
        With g_tblHoliParamater(g_intSeq)
            Call GP_SetGetsuCombo(.Getsu)
            LBL_GETSU.Text = .Getsu
            LBL_SEQ.Text = g_intSeq
            Call GP_SetSyoriKbnCombo(.SyoriKbn)
            NUD_HIDUKE.Value = .Hiduke
            CBO_FURIKAE_KBN.SelectedIndex = .FurikaeKbn
            NUD_SYUSU.Value = .HmSyusu
            CBO_YOBI.SelectedIndex = .HmYobi
            TXT_SYUKU_NM.Text = .SyukuNm
            NUD_STR_YEAR.Value = .StrYear
            NUD_END_YEAR.Value = .EndYear
        End With
        ' 処理区分による表示制御
        Call GP_SetSyoriKbn()
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_SetSyoriKbn
    '* 機能　　：処理区分による表示制御
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_SetSyoriKbn()
        '-------------------------------------------------------------------------------------------
        Select Case FP_GetSyoriKbnCombo()
            Case 0, 2                                           ' 固定日
                GRB_KOTEIBI.Enabled = True
                GRB_HM.Enabled = False
            Case 1                                              ' HappyMonday
                GRB_KOTEIBI.Enabled = False
                GRB_HM.Enabled = True
            Case Else                                           ' 特殊
                GRB_KOTEIBI.Enabled = False
                GRB_HM.Enabled = False
        End Select
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_SetFormCondition
    '* 機能　　：フォームの状態制御
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_SetFormCondition()
        '-------------------------------------------------------------------------------------------
        ' 更新モードによる分岐
        Select Case g_intUpdateMode
            Case 0                          ' 参照モード(本処理では発生しない)
                ' 入力コントロール表示制御
                Call GP_ControlsChangeVisible(False)
                BTN_OK.Visible = False
                MNU_NEW.Visible = False
                MNU_COPY.Visible = False
                MNU_DEL.Visible = False
                CBO_GETSU.Visible = False
                LBL_SEQ.Visible = True
                LBL_SEQ2.Visible = True
                STS_MODE.Text = g_cnsMODE_DSP
                STS_GUIDE.Text = String.Empty
                '-----------------------------------------------------------------------------------
            Case 1                          ' 新規登録モード
                ' 入力コントロール表示制御
                Call GP_ControlsChangeVisible(True)
                MNU_NEW.Visible = False
                MNU_COPY.Visible = False
                MNU_DEL.Visible = False
                CBO_GETSU.Visible = True
                LBL_SEQ.Visible = False
                LBL_SEQ2.Visible = False
                STS_MODE.Text = g_cnsMODE_NEW
                STS_GUIDE.Text = g_cnsMenuGuideI
                BTN_OK.Visible = True
                '-----------------------------------------------------------------------------------
            Case Else                       ' 更新登録モード
                ' 入力コントロール表示制御
                Call GP_ControlsChangeVisible(True)
                MNU_NEW.Visible = True
                    MNU_DEL.Visible = True
                MNU_COPY.Visible = True
                CBO_GETSU.Visible = False
                LBL_SEQ.Visible = True
                LBL_SEQ2.Visible = True
                STS_MODE.Text = g_cnsMODE_UPD
                STS_GUIDE.Text = g_cnsMenuGuideI
                BTN_OK.Visible = True
                '-----------------------------------------------------------------------------------
        End Select
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_ControlsChangeVisible
    '* 機能　　：入力コントロール表示制御
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 状態(Boolean)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：キーコードを除く各コントロールの表示制御
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_ControlsChangeVisible(ByVal blnJotai As Boolean)
        '-------------------------------------------------------------------------------------------
        CBO_SYORI_KBN.Visible = blnJotai
        NUD_HIDUKE.Visible = blnJotai
        CBO_FURIKAE_KBN.Visible = blnJotai
        NUD_SYUSU.Visible = blnJotai
        CBO_YOBI.Visible = blnJotai
        TXT_SYUKU_NM.Visible = blnJotai
        NUD_STR_YEAR.Visible = blnJotai
        NUD_END_YEAR.Visible = blnJotai
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_ClearForm
    '* 機能　　：内容表示クリア
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_ClearForm()
        '-------------------------------------------------------------------------------------------
        CBO_SYORI_KBN.SelectedIndex = -1
        NUD_HIDUKE.Value = 0
        CBO_FURIKAE_KBN.SelectedIndex = -1
        NUD_SYUSU.Value = 0
        CBO_YOBI.SelectedIndex = -1
        TXT_SYUKU_NM.Text = String.Empty
        NUD_STR_YEAR.Value = 0
        NUD_END_YEAR.Value = 9999
        LBL_SEQ.Text = String.Empty
    End Sub

    '***********************************************************************************************
    '* 処理名　：FP_GetGetsuCombo
    '* 機能　　：月コンボから値を取り出す
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：月(Integer)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_GetGetsuCombo() As Integer
        '-------------------------------------------------------------------------------------------
        Dim intSelectedIndex As Integer = CBO_GETSU.SelectedIndex   ' SelectedIndex
        ' 選択されているか
        If intSelectedIndex >= 0 Then
            Return intSelectedIndex + 1
        Else
            Return -1
        End If
    End Function

    '***********************************************************************************************
    '* 処理名　：GP_SetGetsuCombo
    '* 機能　　：月コンボに値をセット
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 月(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_SetGetsuCombo(ByVal intGetsu As Integer)
        '-------------------------------------------------------------------------------------------
        CBO_GETSU.SelectedIndex = intGetsu - 1
    End Sub

    '***********************************************************************************************
    '* 処理名　：FP_GetSyoriKbnCombo
    '* 機能　　：処理区分コンボから値(№)を取り出す
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：処理区分(Integer)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Function FP_GetSyoriKbnCombo() As Integer
        '-------------------------------------------------------------------------------------------
        Dim intSelectedIndex As Integer = CBO_SYORI_KBN.SelectedIndex ' SelectedIndex
        ' 選択されているか
        If intSelectedIndex >= 0 Then
            Return g_tblSyoriKbn(intSelectedIndex)
        Else
            Return -1
        End If
    End Function

    '***********************************************************************************************
    '* 処理名　：GP_SetSyoriKbnCombo
    '* 機能　　：処理区分コンボに値(№)をセット
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 月(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_SetSyoriKbnCombo(ByVal intSyoriKbn As Integer)
        '-------------------------------------------------------------------------------------------
        Dim intSelectedIndex As Integer = -1                        ' SelectedIndex
        Dim intIx As Integer = 0                                    ' テーブルINDEX
        ' テーブルを探索
        Do While intIx <= g_tblSyoriKbn.GetUpperBound(0)
            ' 処理区分発見か
            If g_tblSyoriKbn(intIx) = intSyoriKbn Then
                intSelectedIndex = intIx
                Exit Do
            End If
            ' 次へ
            intIx += 1
        Loop
        CBO_SYORI_KBN.SelectedIndex = intSelectedIndex
    End Sub

    '***********************************************************************************************
    ' ■■■ 以下は親フォームとの受け渡しプロパティ ■■■
    '***********************************************************************************************
    ' 処理モード(Integer)
    '-----------------------------------------------------------------------------------------------
    Friend WriteOnly Property prpUpdateMode() As Integer
        Set(ByVal value As Integer)
            g_intUpdateMode = value
        End Set
    End Property

    '===============================================================================================
    ' SEQ(Integer)
    '-----------------------------------------------------------------------------------------------
    Friend WriteOnly Property prpSeq() As Integer
        Set(ByVal value As Integer)
            g_intSeq = value
        End Set
    End Property

    '===============================================================================================
    ' 更新有無(戻り値)
    '-----------------------------------------------------------------------------------------------
    Friend ReadOnly Property prpUpdateResult() As Integer
        Get
            Return g_intUpdateResult
        End Get
    End Property

    '---------------------------------------<< End of Source >>-------------------------------------
End Class

