﻿'***************************************************************************************************
'   年間カレンダーの作成２                              frmMakeYearyCalendar2(Form)
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev     変更履歴内容-------------------------------------------------------------------->
' 18/09/14(1.0.0.0)新規作成
' 18/09/18(1.0.1.0)カレンダークラスのバージョン表示対応(クラス参照をモジュールに移動)
' 18/12/16(1.0.2.0)｢祝日変更｣メニューの追加､フォーム表示位置保持機能の追加
'***************************************************************************************************
Imports System.IO
Public Class frmMakeYearyCalendar2
    '===============================================================================================
    Private Const g_cnsExcelTemplate As String = "YearyCalendar1.xltx"
    ' クリアするセル範囲
    Private Const g_cnsClearRange As String = "$B$1:$D$1,$B$6:$Z$16,$B$22:$Z$33,$B$38:$Z$49,$B$54:$Z$65"
    '-----------------------------------------------------------------------------------------------
    ' ダブルクリック抑制クラス(初期化はNewプロシージャで行なう)
    Private g_objOmitDoubleClick As clsOmitDoubleClick2 ' ダブルクリック抑制クラス
    Private g_blnEnabledODCClass As Boolean = False     ' ダブルクリック抑制クラス利用可
    '-----------------------------------------------------------------------------------------------
    ' リスト上限年をシステム日の年+n年まで可とするパラメータ
    Private Const g_cnsMaxAddYear As Integer = 3        ' リスト上限年加算値

    '***********************************************************************************************
    ' ■■■ 初期化 ■■■
    '***********************************************************************************************
    '* 処理名　：New
    '* 機能　　：全体初期化
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(なし)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年09月14日
    '* 作成者　：井上　治
    '* 更新日　：2018年09月18日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Public Sub New()
        '-------------------------------------------------------------------------------------------
        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()
        ' フォームデザイナモード時は以下をスキップする
        If Me.DesignMode Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック抑制クラスの初期化
        g_objOmitDoubleClick = New clsOmitDoubleClick2(Me, True, g_blnEnabledODCClass)
        ' カレンダー関連クラスの初期化
        g_clsAboutCalendar = New clsAboutCalendar2VB
        '-------------------------------------------------------------------------------------------
        ' DPI値によるフォーム／コントロールの位置・サイズ調整
        Call modFormDpiMgr1.GP_AdjustFormSizeByDpi3(Me)
    End Sub

    '***********************************************************************************************
    ' ■■■ Formイベント ■■■
    '***********************************************************************************************
    '* 処理名　：Form_FormClosed
    '* 機能　　：フォームイベント(FormClosed)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub Form_FormClosed(ByVal sender As Object, _
                                ByVal e As FormClosedEventArgs) Handles Me.FormClosed
        '-------------------------------------------------------------------------------------------
        ' フォーム位置を退避
        With My.Settings
            .START_FormLocation = Me.Location
            ' 設定を保存
            .Save()
        End With
    End Sub

    '***********************************************************************************************
    '* 処理名　：Form_Load
    '* 機能　　：フォームイベント(Load)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年09月14日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub Form_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        '-------------------------------------------------------------------------------------------
        Dim intCurrYear As Integer = Today.Year                     ' 現在年
        Dim intMaxYear As Integer = intCurrYear + g_cnsMaxAddYear   ' 最大年
        Dim intYear As Integer = 2000                               ' 開始年
        Dim intCurrIx As Integer = 0                                ' 現在年INDEX
        Dim intIx As Integer = -1                                   ' テーブルINDEX
        Dim tblItem() As String
        ReDim tblItem(intIx)
        '-------------------------------------------------------------------------------------------
        ' フォーム位置制御
        Call GP_FormAdjustLocation1(Me, My.Settings.START_FormLocation, False, Me.Size)
        '-------------------------------------------------------------------------------------------
        ' リスト用テーブル作成
        Do While intYear <= intMaxYear
            intIx += 1
            ReDim Preserve tblItem(intIx)
            tblItem(intIx) = intYear.ToString & "年"
            ' 現在年が見つかったらINDEXを取得
            If intYear = intCurrYear Then
                intCurrIx = intIx
            End If
            ' 次の年へ
            intYear += 1
        Loop
        ' リストボックスに年リストを作成
        With LST_START
            With .Items
                If .Count <> 0 Then .Clear()
                .AddRange(tblItem)
            End With
            .SelectedIndex = 0
        End With
        With LST_END
            With .Items
                If .Count <> 0 Then .Clear()
                .AddRange(tblItem)
            End With
            .SelectedIndex = intCurrIx + 1
        End With
        ' カレンダーVer表示
        STS_GUIDE.Text = "Ver" & My.Application.Info.Version.ToString
    End Sub

    '***********************************************************************************************
    '* 処理名　：Form_Shown
    '* 機能　　：フォームイベント(Shown)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(デフォルト)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年09月14日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub Form_Shown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Shown
        '-------------------------------------------------------------------------------------------
        Me.Activate()
        Me.Refresh()
        ' 初期表示動作完了
        g_objOmitDoubleClick.Shown = True
    End Sub

    '***********************************************************************************************
    ' ■■■ コントロールイベント ■■■
    '***********************************************************************************************
    '* 処理名　：BTN_OK_Click
    '* 機能　　：「作成」ボタンイベント(Click)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(既定)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年09月14日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub BTN_OK_Click(ByVal sender As Object, _
                             ByVal e As System.EventArgs) Handles BTN_OK.Click
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック等の多重操作を抑制
        If Not g_blnEnabledODCClass OrElse g_objOmitDoubleClick.CheckDoubleClick() Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' 指定値の取り出し
        Dim intStrYear As Integer = Integer.Parse(LST_START.Text.Substring(0, 4)) ' 開始年
        Dim intEndYear As Integer = Integer.Parse(LST_END.Text.Substring(0, 4))   ' 終了年
        ' 逆転はエラー
        If intEndYear < intStrYear Then
            MessageBox2.DialogShow(Me, _
                                   "「開始年」「終了年」が逆転しています。", _
                                   g_cnsProgTitle, _
                                   MessageBoxButtons.OK, _
                                   MessageBoxIcon.Exclamation)
        End If
        '-------------------------------------------------------------------------------------------
        BTN_OK.Focus()
        Application.DoEvents()
        ' Excel出力クラスの初期化
        Using clsExcel = New clsAboutExcel4(Me)
            ' ｢登録｣ボタンClick
            Call GP_OK_Click(clsExcel, intStrYear, intEndYear)
        End Using
        ' カレンダーVer表示
        STS_GUIDE.Text = "Ver" & My.Application.Info.Version.ToString
    End Sub

    '***********************************************************************************************
    '* 処理名　：MNU_CHG_HOLIPARA_Click
    '* 機能　　：「祝日変更」メニューイベント(Click)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(既定)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年12月16日
    '* 作成者　：井上　治
    '* 更新日　：2018年12月16日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub MNU_CHG_HOLIPARA_Click(ByVal sender As Object, _
                                       ByVal e As System.EventArgs) Handles MNU_CHG_HOLIPARA.Click
        '-------------------------------------------------------------------------------------------
        ' ダブルクリック等の多重操作を抑制
        If Not g_blnEnabledODCClass OrElse g_objOmitDoubleClick.CheckDoubleClick() Then Exit Sub
        '-------------------------------------------------------------------------------------------
        ' 祝日パラメータ一覧フォームの起動
        Using objHoliPara As frmSetHoliParam = New frmSetHoliParam
            objHoliPara.ShowDialog(Me)
        End Using
    End Sub

    '***********************************************************************************************
    ' ■■■ サブ処理 ■■■
    '***********************************************************************************************
    '* 処理名　：GP_OK_Click
    '* 機能　　：｢作成｣ボタンClickサブ
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = Excel出力クラス(Object)
    '* 　　　　　Arg2 = 開始年(Integer)
    '* 　　　　　Arg3 = 終了年(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年09月14日
    '* 作成者　：井上　治
    '* 更新日　：2018年09月18日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_OK_Click(ByRef clsExcel As clsAboutExcel4, _
                            ByVal intStrYear As Integer, _
                            ByVal intEndYear As Integer)
        '-------------------------------------------------------------------------------------------
        Dim objExcel As Object = Nothing                            ' Excel.Application
        Dim objWBK As Object = Nothing                              ' Excel.Workbook
        Dim intCntSheet As Integer = 1                              ' シート数カウンタ
        Dim strMSG As String = String.Empty                         ' エラーメッセージ
        Dim strPathname As String = My.Application.Info.DirectoryPath ' アプリケーションフォルダ
        Dim strExcelFilename As String = Path.Combine(strPathname, g_cnsExcelTemplate) ' フルパス名
        '-------------------------------------------------------------------------------------------
        ' Excelを起動し、テンプレートから新規ワークブックを開く
        If Not clsExcel.GetWorkbook(strExcelFilename, objExcel, objWBK) Then Exit Sub
        ' 画面描画停止
        Call clsExcel.ExcelStopScreenUpdate(objExcel)
        '-------------------------------------------------------------------------------------------
        Dim objSh0 As Object = objWBK.Worksheets(1)                 ' 原紙シート
        ' 年をループさせて各年の年間カレンダーを作成
        For intYear As Integer = intStrYear To intEndYear
            Dim objShP As Object = objWBK.Worksheets(intCntSheet)   ' 直前シート
            ' 原紙シートをコピー
            objSh0.Copy(After:=objShP)
            intCntSheet += 1
            Dim objSh As Object = objWBK.Worksheets(intCntSheet)    ' 現在シート
            ' シート名は｢nnnn年｣
            objSh.Name = intYear.ToString & "年"
            ' 年間カレンダー作成
            Call GP_MakeYearyCalendar(objSh, intYear)
        Next intYear
        ' ｢原紙｣シートは非表示にする
        objSh0.Visible = xlSheetVeryHidden
        Call clsExcel.ReleaseComObject(objSh0)
        objWBK.Saved = True
        '-------------------------------------------------------------------------------------------
        ' Excelを表示して終了
        Call clsExcel.SuspendExcelProc(objExcel, objWBK, True)
    End Sub

    '***********************************************************************************************
    '* 処理名　：GP_MakeYearyCalendar
    '* 機能　　：年間カレンダー作成(処理本体)
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 処理ワークシート(Object)
    '* 　　　　　Arg2 = 作成年(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2018年09月14日
    '* 作成者　：井上　治
    '* 更新日　：2018年09月18日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_MakeYearyCalendar(ByRef objSh As Object, ByVal intY As Integer)
        '-------------------------------------------------------------------------------------------
        Dim tblRow() As Integer = New Integer() {6, 6, 6, 22, 22, 22, 38, 38, 38, 54, 54, 54}
        Dim tblCol() As Integer = New Integer() {2, 11, 20, 2, 11, 20, 2, 11, 20, 2, 11, 20}
        Dim tblCalendar() As clsTypClassCalendar2 = Nothing
        '-------------------------------------------------------------------------------------------
        ' カレンダーシート
        With objSh
            ' 範囲クリア
            .Range(g_cnsClearRange).ClearContents()
            ' 年表示
            .Cells(1, 2).Value = intY
            ' 月を巡回
            For intM As Integer = 1 To 12
                '----------------------------------------------
                STS_GUIDE.Text = intY.ToString & "年" & intM.ToString("00") & "月"
                StatusStrip1.Refresh()
                '----------------------------------------------
                ' カレンダーテーブル作成(1ヶ月分)
                Call g_clsAboutCalendar.GetCalendarTable1(intY, intM, tblCalendar)
                '----------------------------------------------
                Dim intIx As Integer = 0                            ' テーブルINDEX
                Dim intIxR As Integer = 0                           ' テーブルINDEX(行方向)
                Dim intIxC As Integer = tblCalendar(intIx).Yobi     ' テーブルINDEX(列方向)
                Dim tblCalRange(11, 6) As Object                    ' セル範囲に貼り付ける矩形
                ' カレンダーテーブルを巡回
                Do While intIx <= tblCalendar.GetUpperBound(0)
                    tblCalRange(intIxR, intIxC) = tblCalendar(intIx).Hiduke.Day
                    tblCalRange(intIxR + 1, intIxC) = tblCalendar(intIx).Name
                    ' 次の日へ
                    intIx += 1
                    intIxC += 1
                    ' 曜日飽和なら週を加算
                    If intIxC > 6 Then
                        intIxR += +2
                        intIxC = 0
                    End If
                Loop
                '-----------------------------------------------
                Dim intRow As Integer = tblRow(intM - 1)            ' 行INDEX
                Dim intCol As Integer = tblCol(intM - 1)            ' 列INDEX
                ' セル範囲に貼り付け
                .Range(.Cells(intRow, intCol), .Cells(intRow + 11, intCol + 6)).Value = tblCalRange
            Next intM
            .Range("$A$1").Select()
        End With
    End Sub

    '----------------------------------------<< End of Source >>------------------------------------
End Class
